# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/noplay/code/gns3/gns3-gui/gns3/ui/appliance_wizard.ui'
#
# Created by: PyQt5 UI code generator 5.5.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ApplianceWizard(object):
    def setupUi(self, ApplianceWizard):
        ApplianceWizard.setObjectName("ApplianceWizard")
        ApplianceWizard.resize(704, 504)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(ApplianceWizard.sizePolicy().hasHeightForWidth())
        ApplianceWizard.setSizePolicy(sizePolicy)
        ApplianceWizard.setMaximumSize(QtCore.QSize(2000, 2000))
        ApplianceWizard.setModal(True)
        ApplianceWizard.setWizardStyle(QtWidgets.QWizard.ClassicStyle)
        ApplianceWizard.setOptions(QtWidgets.QWizard.NoBackButtonOnStartPage)
        self.uiInfoWizardPage = QtWidgets.QWizardPage()
        self.uiInfoWizardPage.setSubTitle("")
        self.uiInfoWizardPage.setObjectName("uiInfoWizardPage")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.uiInfoWizardPage)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.uiInfoTreeWidget = QtWidgets.QTreeWidget(self.uiInfoWizardPage)
        self.uiInfoTreeWidget.setAlternatingRowColors(False)
        self.uiInfoTreeWidget.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.uiInfoTreeWidget.setTextElideMode(QtCore.Qt.ElideRight)
        self.uiInfoTreeWidget.setRootIsDecorated(False)
        self.uiInfoTreeWidget.setWordWrap(False)
        self.uiInfoTreeWidget.setHeaderHidden(True)
        self.uiInfoTreeWidget.setObjectName("uiInfoTreeWidget")
        item_0 = QtWidgets.QTreeWidgetItem(self.uiInfoTreeWidget)
        item_0 = QtWidgets.QTreeWidgetItem(self.uiInfoTreeWidget)
        item_0 = QtWidgets.QTreeWidgetItem(self.uiInfoTreeWidget)
        item_0 = QtWidgets.QTreeWidgetItem(self.uiInfoTreeWidget)
        item_0 = QtWidgets.QTreeWidgetItem(self.uiInfoTreeWidget)
        item_0 = QtWidgets.QTreeWidgetItem(self.uiInfoTreeWidget)
        self.uiInfoTreeWidget.header().setVisible(False)
        self.gridLayout_4.addWidget(self.uiInfoTreeWidget, 1, 0, 1, 1)
        self.uiDescriptionLabel = QtWidgets.QLabel(self.uiInfoWizardPage)
        self.uiDescriptionLabel.setScaledContents(False)
        self.uiDescriptionLabel.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.uiDescriptionLabel.setWordWrap(True)
        self.uiDescriptionLabel.setObjectName("uiDescriptionLabel")
        self.gridLayout_4.addWidget(self.uiDescriptionLabel, 0, 0, 1, 1)
        ApplianceWizard.addPage(self.uiInfoWizardPage)
        self.uiServerWizardPage = QtWidgets.QWizardPage()
        self.uiServerWizardPage.setObjectName("uiServerWizardPage")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.uiServerWizardPage)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label = QtWidgets.QLabel(self.uiServerWizardPage)
        self.label.setText("")
        self.label.setObjectName("label")
        self.verticalLayout_2.addWidget(self.label)
        self.uiServerTypeGroupBox = QtWidgets.QGroupBox(self.uiServerWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiServerTypeGroupBox.sizePolicy().hasHeightForWidth())
        self.uiServerTypeGroupBox.setSizePolicy(sizePolicy)
        self.uiServerTypeGroupBox.setMinimumSize(QtCore.QSize(0, 161))
        self.uiServerTypeGroupBox.setObjectName("uiServerTypeGroupBox")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.uiServerTypeGroupBox)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.label_2 = QtWidgets.QLabel(self.uiServerTypeGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        self.label_2.setMinimumSize(QtCore.QSize(0, 17))
        self.label_2.setObjectName("label_2")
        self.verticalLayout_4.addWidget(self.label_2)
        self.uiRemoteRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRemoteRadioButton.sizePolicy().hasHeightForWidth())
        self.uiRemoteRadioButton.setSizePolicy(sizePolicy)
        self.uiRemoteRadioButton.setMinimumSize(QtCore.QSize(0, 20))
        self.uiRemoteRadioButton.setChecked(True)
        self.uiRemoteRadioButton.setObjectName("uiRemoteRadioButton")
        self.verticalLayout_4.addWidget(self.uiRemoteRadioButton)
        self.uiVMRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiVMRadioButton.sizePolicy().hasHeightForWidth())
        self.uiVMRadioButton.setSizePolicy(sizePolicy)
        self.uiVMRadioButton.setMinimumSize(QtCore.QSize(0, 20))
        self.uiVMRadioButton.setObjectName("uiVMRadioButton")
        self.verticalLayout_4.addWidget(self.uiVMRadioButton)
        self.uiLocalRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiLocalRadioButton.sizePolicy().hasHeightForWidth())
        self.uiLocalRadioButton.setSizePolicy(sizePolicy)
        self.uiLocalRadioButton.setMinimumSize(QtCore.QSize(0, 20))
        self.uiLocalRadioButton.setObjectName("uiLocalRadioButton")
        self.verticalLayout_4.addWidget(self.uiLocalRadioButton)
        spacerItem = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem)
        self.verticalLayout_2.addWidget(self.uiServerTypeGroupBox)
        self.uiRemoteServersGroupBox = QtWidgets.QGroupBox(self.uiServerWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRemoteServersGroupBox.sizePolicy().hasHeightForWidth())
        self.uiRemoteServersGroupBox.setSizePolicy(sizePolicy)
        self.uiRemoteServersGroupBox.setObjectName("uiRemoteServersGroupBox")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.uiRemoteServersGroupBox)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.uiRemoteServersComboBox = QtWidgets.QComboBox(self.uiRemoteServersGroupBox)
        self.uiRemoteServersComboBox.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRemoteServersComboBox.sizePolicy().hasHeightForWidth())
        self.uiRemoteServersComboBox.setSizePolicy(sizePolicy)
        self.uiRemoteServersComboBox.setObjectName("uiRemoteServersComboBox")
        self.gridLayout_7.addWidget(self.uiRemoteServersComboBox, 2, 1, 1, 1)
        self.uiLoadBalanceCheckBox = QtWidgets.QCheckBox(self.uiRemoteServersGroupBox)
        self.uiLoadBalanceCheckBox.setChecked(True)
        self.uiLoadBalanceCheckBox.setObjectName("uiLoadBalanceCheckBox")
        self.gridLayout_7.addWidget(self.uiLoadBalanceCheckBox, 1, 0, 1, 2)
        self.uiRemoteServersLabel = QtWidgets.QLabel(self.uiRemoteServersGroupBox)
        self.uiRemoteServersLabel.setObjectName("uiRemoteServersLabel")
        self.gridLayout_7.addWidget(self.uiRemoteServersLabel, 2, 0, 1, 1)
        self.verticalLayout_2.addWidget(self.uiRemoteServersGroupBox)
        ApplianceWizard.addPage(self.uiServerWizardPage)
        self.uiCheckServerWizardPage = QtWidgets.QWizardPage()
        self.uiCheckServerWizardPage.setObjectName("uiCheckServerWizardPage")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.uiCheckServerWizardPage)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.uiCheckServerLabel = QtWidgets.QLabel(self.uiCheckServerWizardPage)
        self.uiCheckServerLabel.setWordWrap(True)
        self.uiCheckServerLabel.setObjectName("uiCheckServerLabel")
        self.verticalLayout_3.addWidget(self.uiCheckServerLabel)
        ApplianceWizard.addPage(self.uiCheckServerWizardPage)
        self.uiFilesWizardPage = QtWidgets.QWizardPage()
        self.uiFilesWizardPage.setObjectName("uiFilesWizardPage")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.uiFilesWizardPage)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label_3 = QtWidgets.QLabel(self.uiFilesWizardPage)
        self.label_3.setTextFormat(QtCore.Qt.RichText)
        self.label_3.setAlignment(QtCore.Qt.AlignJustify|QtCore.Qt.AlignVCenter)
        self.label_3.setWordWrap(True)
        self.label_3.setObjectName("label_3")
        self.verticalLayout.addWidget(self.label_3)
        self.uiApplianceVersionTreeWidget = QtWidgets.QTreeWidget(self.uiFilesWizardPage)
        self.uiApplianceVersionTreeWidget.setIndentation(20)
        self.uiApplianceVersionTreeWidget.setObjectName("uiApplianceVersionTreeWidget")
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.uiApplianceVersionTreeWidget.headerItem().setFont(0, font)
        item_0 = QtWidgets.QTreeWidgetItem(self.uiApplianceVersionTreeWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        item_0.setFont(0, font)
        item_1 = QtWidgets.QTreeWidgetItem(item_0)
        item_0 = QtWidgets.QTreeWidgetItem(self.uiApplianceVersionTreeWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        item_0.setFont(0, font)
        item_1 = QtWidgets.QTreeWidgetItem(item_0)
        item_0 = QtWidgets.QTreeWidgetItem(self.uiApplianceVersionTreeWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        item_0.setFont(0, font)
        item_1 = QtWidgets.QTreeWidgetItem(item_0)
        item_1 = QtWidgets.QTreeWidgetItem(item_0)
        self.uiApplianceVersionTreeWidget.header().setDefaultSectionSize(120)
        self.uiApplianceVersionTreeWidget.header().setMinimumSectionSize(20)
        self.verticalLayout.addWidget(self.uiApplianceVersionTreeWidget)
        self.uiExplainDownloadLabel = QtWidgets.QLabel(self.uiFilesWizardPage)
        self.uiExplainDownloadLabel.setWordWrap(True)
        self.uiExplainDownloadLabel.setObjectName("uiExplainDownloadLabel")
        self.verticalLayout.addWidget(self.uiExplainDownloadLabel)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.uiImportPushButton = QtWidgets.QPushButton(self.uiFilesWizardPage)
        self.uiImportPushButton.setObjectName("uiImportPushButton")
        self.horizontalLayout.addWidget(self.uiImportPushButton)
        self.uiDownloadPushButton = QtWidgets.QPushButton(self.uiFilesWizardPage)
        self.uiDownloadPushButton.setObjectName("uiDownloadPushButton")
        self.horizontalLayout.addWidget(self.uiDownloadPushButton)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.uiCreateVersionPushButton = QtWidgets.QPushButton(self.uiFilesWizardPage)
        self.uiCreateVersionPushButton.setObjectName("uiCreateVersionPushButton")
        self.horizontalLayout.addWidget(self.uiCreateVersionPushButton)
        self.uiRefreshPushButton = QtWidgets.QPushButton(self.uiFilesWizardPage)
        self.uiRefreshPushButton.setObjectName("uiRefreshPushButton")
        self.horizontalLayout.addWidget(self.uiRefreshPushButton)
        self.verticalLayout.addLayout(self.horizontalLayout)
        ApplianceWizard.addPage(self.uiFilesWizardPage)
        self.uiQemuWizardPage = QtWidgets.QWizardPage()
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiQemuWizardPage.sizePolicy().hasHeightForWidth())
        self.uiQemuWizardPage.setSizePolicy(sizePolicy)
        self.uiQemuWizardPage.setObjectName("uiQemuWizardPage")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.uiQemuWizardPage)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.uiQemuListLabel = QtWidgets.QLabel(self.uiQemuWizardPage)
        self.uiQemuListLabel.setObjectName("uiQemuListLabel")
        self.horizontalLayout_2.addWidget(self.uiQemuListLabel)
        self.uiQemuListComboBox = QtWidgets.QComboBox(self.uiQemuWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiQemuListComboBox.sizePolicy().hasHeightForWidth())
        self.uiQemuListComboBox.setSizePolicy(sizePolicy)
        self.uiQemuListComboBox.setObjectName("uiQemuListComboBox")
        self.horizontalLayout_2.addWidget(self.uiQemuListComboBox)
        ApplianceWizard.addPage(self.uiQemuWizardPage)
        self.uiSummaryWizardPage = QtWidgets.QWizardPage()
        self.uiSummaryWizardPage.setObjectName("uiSummaryWizardPage")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.uiSummaryWizardPage)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.uiSummaryTreeWidget = QtWidgets.QTreeWidget(self.uiSummaryWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSummaryTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiSummaryTreeWidget.setSizePolicy(sizePolicy)
        self.uiSummaryTreeWidget.setAlternatingRowColors(False)
        self.uiSummaryTreeWidget.setRootIsDecorated(False)
        self.uiSummaryTreeWidget.setUniformRowHeights(False)
        self.uiSummaryTreeWidget.setItemsExpandable(False)
        self.uiSummaryTreeWidget.setAnimated(False)
        self.uiSummaryTreeWidget.setAllColumnsShowFocus(False)
        self.uiSummaryTreeWidget.setWordWrap(False)
        self.uiSummaryTreeWidget.setHeaderHidden(True)
        self.uiSummaryTreeWidget.setColumnCount(2)
        self.uiSummaryTreeWidget.setObjectName("uiSummaryTreeWidget")
        item_0 = QtWidgets.QTreeWidgetItem(self.uiSummaryTreeWidget)
        item_0 = QtWidgets.QTreeWidgetItem(self.uiSummaryTreeWidget)
        item_0 = QtWidgets.QTreeWidgetItem(self.uiSummaryTreeWidget)
        item_0 = QtWidgets.QTreeWidgetItem(self.uiSummaryTreeWidget)
        item_0 = QtWidgets.QTreeWidgetItem(self.uiSummaryTreeWidget)
        item_0 = QtWidgets.QTreeWidgetItem(self.uiSummaryTreeWidget)
        self.uiSummaryTreeWidget.header().setDefaultSectionSize(150)
        self.uiSummaryTreeWidget.header().setMinimumSectionSize(20)
        self.uiSummaryTreeWidget.header().setStretchLastSection(True)
        self.gridLayout_2.addWidget(self.uiSummaryTreeWidget, 0, 0, 1, 1)
        ApplianceWizard.addPage(self.uiSummaryWizardPage)
        self.uiUsageWizardPage = QtWidgets.QWizardPage()
        self.uiUsageWizardPage.setObjectName("uiUsageWizardPage")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.uiUsageWizardPage)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.uiUsageTextEdit = QtWidgets.QTextEdit(self.uiUsageWizardPage)
        self.uiUsageTextEdit.setObjectName("uiUsageTextEdit")
        self.gridLayout_3.addWidget(self.uiUsageTextEdit, 0, 0, 1, 1)
        ApplianceWizard.addPage(self.uiUsageWizardPage)

        self.retranslateUi(ApplianceWizard)
        QtCore.QMetaObject.connectSlotsByName(ApplianceWizard)

    def retranslateUi(self, ApplianceWizard):
        _translate = QtCore.QCoreApplication.translate
        ApplianceWizard.setWindowTitle(_translate("ApplianceWizard", "Add appliance"))
        self.uiInfoWizardPage.setTitle(_translate("ApplianceWizard", "Cisco NX-OSv"))
        self.uiInfoTreeWidget.headerItem().setText(0, _translate("ApplianceWizard", "1"))
        self.uiInfoTreeWidget.headerItem().setText(1, _translate("ApplianceWizard", "2"))
        __sortingEnabled = self.uiInfoTreeWidget.isSortingEnabled()
        self.uiInfoTreeWidget.setSortingEnabled(False)
        self.uiInfoTreeWidget.topLevelItem(0).setText(0, _translate("ApplianceWizard", "Category:"))
        self.uiInfoTreeWidget.topLevelItem(0).setText(1, _translate("ApplianceWizard", "router"))
        self.uiInfoTreeWidget.topLevelItem(1).setText(0, _translate("ApplianceWizard", "Product:"))
        self.uiInfoTreeWidget.topLevelItem(1).setText(1, _translate("ApplianceWizard", "NX-OSv"))
        self.uiInfoTreeWidget.topLevelItem(2).setText(0, _translate("ApplianceWizard", "Vendor:"))
        self.uiInfoTreeWidget.topLevelItem(2).setText(1, _translate("ApplianceWizard", "Cisco"))
        self.uiInfoTreeWidget.topLevelItem(3).setText(0, _translate("ApplianceWizard", "Status:"))
        self.uiInfoTreeWidget.topLevelItem(3).setText(1, _translate("ApplianceWizard", "stable"))
        self.uiInfoTreeWidget.topLevelItem(4).setText(0, _translate("ApplianceWizard", "Maintainer:"))
        self.uiInfoTreeWidget.topLevelItem(4).setText(1, _translate("ApplianceWizard", "The GNS3 team"))
        self.uiInfoTreeWidget.setSortingEnabled(__sortingEnabled)
        self.uiDescriptionLabel.setText(_translate("ApplianceWizard", "NX-OSv is a reference platform for an implementation of the Cisco Nexus operating system, based on the Nexus 7000-series platforms, running as a full virtual machine on a hypervisor."))
        self.uiServerWizardPage.setTitle(_translate("ApplianceWizard", "Server"))
        self.uiServerWizardPage.setSubTitle(_translate("ApplianceWizard", "Please choose a server type to run your new Appliance."))
        self.uiServerTypeGroupBox.setTitle(_translate("ApplianceWizard", "Server type"))
        self.label_2.setText(_translate("ApplianceWizard", "The grayed server types are not supported or configured."))
        self.uiRemoteRadioButton.setText(_translate("ApplianceWizard", "Run the appliance on a remote computer"))
        self.uiVMRadioButton.setText(_translate("ApplianceWizard", "Run the appliance on the GNS3 VM (recommended)"))
        self.uiLocalRadioButton.setText(_translate("ApplianceWizard", "Run the appliance on your local computer"))
        self.uiRemoteServersGroupBox.setTitle(_translate("ApplianceWizard", "Remote servers"))
        self.uiLoadBalanceCheckBox.setText(_translate("ApplianceWizard", "Load balance across all available remote servers"))
        self.uiRemoteServersLabel.setText(_translate("ApplianceWizard", "Run on server:"))
        self.uiCheckServerLabel.setText(_translate("ApplianceWizard", "Please wait while checking server capacities..."))
        self.uiFilesWizardPage.setTitle(_translate("ApplianceWizard", "Required files"))
        self.uiFilesWizardPage.setSubTitle(_translate("ApplianceWizard", "The following files are required to install NX-OS"))
        self.label_3.setText(_translate("ApplianceWizard", "<html><head/><body><p>Click on a version to see the required files and import the file from your computer.<br/>GNS3 is looking for files in your downloads directory and in the GNS3 images directory.</p></body></html>"))
        self.uiApplianceVersionTreeWidget.headerItem().setText(0, _translate("ApplianceWizard", "Version"))
        self.uiApplianceVersionTreeWidget.headerItem().setText(1, _translate("ApplianceWizard", "Filename"))
        self.uiApplianceVersionTreeWidget.headerItem().setText(2, _translate("ApplianceWizard", "Size"))
        self.uiApplianceVersionTreeWidget.headerItem().setText(3, _translate("ApplianceWizard", "Status"))
        self.uiApplianceVersionTreeWidget.headerItem().setText(4, _translate("ApplianceWizard", "File version"))
        self.uiApplianceVersionTreeWidget.headerItem().setText(5, _translate("ApplianceWizard", "MD5"))
        __sortingEnabled = self.uiApplianceVersionTreeWidget.isSortingEnabled()
        self.uiApplianceVersionTreeWidget.setSortingEnabled(False)
        self.uiApplianceVersionTreeWidget.topLevelItem(0).setText(0, _translate("ApplianceWizard", "7.2.0.121"))
        self.uiApplianceVersionTreeWidget.topLevelItem(0).setText(3, _translate("ApplianceWizard", "Missing"))
        self.uiApplianceVersionTreeWidget.topLevelItem(0).child(0).setText(1, _translate("ApplianceWizard", "NXOS-V7.2.0.121.bin"))
        self.uiApplianceVersionTreeWidget.topLevelItem(0).child(0).setText(2, _translate("ApplianceWizard", "345MB"))
        self.uiApplianceVersionTreeWidget.topLevelItem(0).child(0).setText(3, _translate("ApplianceWizard", "Missing"))
        self.uiApplianceVersionTreeWidget.topLevelItem(0).child(0).setText(4, _translate("ApplianceWizard", "7.2.0.121"))
        self.uiApplianceVersionTreeWidget.topLevelItem(0).child(0).setText(5, _translate("ApplianceWizard", "5d41402abc4b2a76b9719d911017c592"))
        self.uiApplianceVersionTreeWidget.topLevelItem(1).setText(0, _translate("ApplianceWizard", "7.3.0.42"))
        self.uiApplianceVersionTreeWidget.topLevelItem(1).setText(3, _translate("ApplianceWizard", "Missing"))
        self.uiApplianceVersionTreeWidget.topLevelItem(1).child(0).setText(1, _translate("ApplianceWizard", "NXOS-V7.3.0.121.bin"))
        self.uiApplianceVersionTreeWidget.topLevelItem(1).child(0).setText(2, _translate("ApplianceWizard", "356MB"))
        self.uiApplianceVersionTreeWidget.topLevelItem(1).child(0).setText(3, _translate("ApplianceWizard", "Missing"))
        self.uiApplianceVersionTreeWidget.topLevelItem(1).child(0).setText(4, _translate("ApplianceWizard", "7.3.0.42"))
        self.uiApplianceVersionTreeWidget.topLevelItem(1).child(0).setText(5, _translate("ApplianceWizard", "5d41402abc4b2a76b9719d911017c592"))
        self.uiApplianceVersionTreeWidget.topLevelItem(2).setText(0, _translate("ApplianceWizard", "8.0"))
        self.uiApplianceVersionTreeWidget.topLevelItem(2).setText(3, _translate("ApplianceWizard", "Available"))
        self.uiApplianceVersionTreeWidget.topLevelItem(2).child(0).setText(1, _translate("ApplianceWizard", "NX-diskA"))
        self.uiApplianceVersionTreeWidget.topLevelItem(2).child(0).setText(2, _translate("ApplianceWizard", "12MB"))
        self.uiApplianceVersionTreeWidget.topLevelItem(2).child(0).setText(3, _translate("ApplianceWizard", "Available"))
        self.uiApplianceVersionTreeWidget.topLevelItem(2).child(0).setText(4, _translate("ApplianceWizard", "8.0"))
        self.uiApplianceVersionTreeWidget.topLevelItem(2).child(0).setText(5, _translate("ApplianceWizard", "5d41402abc4b2a76b9719d911017c592"))
        self.uiApplianceVersionTreeWidget.topLevelItem(2).child(1).setText(1, _translate("ApplianceWizard", "NX-diskB"))
        self.uiApplianceVersionTreeWidget.topLevelItem(2).child(1).setText(2, _translate("ApplianceWizard", "400MB"))
        self.uiApplianceVersionTreeWidget.topLevelItem(2).child(1).setText(3, _translate("ApplianceWizard", "Available"))
        self.uiApplianceVersionTreeWidget.topLevelItem(2).child(1).setText(4, _translate("ApplianceWizard", "8.0"))
        self.uiApplianceVersionTreeWidget.topLevelItem(2).child(1).setText(5, _translate("ApplianceWizard", "5d41402abc4b2a76b9719d911017c592"))
        self.uiApplianceVersionTreeWidget.setSortingEnabled(__sortingEnabled)
        self.uiExplainDownloadLabel.setText(_translate("ApplianceWizard", "Click on a the download button to access to a location where you can download the file."))
        self.uiImportPushButton.setText(_translate("ApplianceWizard", "&Import"))
        self.uiDownloadPushButton.setText(_translate("ApplianceWizard", "&Download"))
        self.uiCreateVersionPushButton.setText(_translate("ApplianceWizard", "Create a new version"))
        self.uiRefreshPushButton.setText(_translate("ApplianceWizard", "Refresh"))
        self.uiQemuWizardPage.setTitle(_translate("ApplianceWizard", "Qemu settings"))
        self.uiQemuWizardPage.setSubTitle(_translate("ApplianceWizard", "Please choose the qemu binary that we will use for running this appliance."))
        self.uiQemuListLabel.setText(_translate("ApplianceWizard", "Qemu binary:"))
        self.uiSummaryWizardPage.setTitle(_translate("ApplianceWizard", "Summary"))
        self.uiSummaryWizardPage.setSubTitle(_translate("ApplianceWizard", "The following settings are going to be used."))
        self.uiSummaryTreeWidget.headerItem().setText(0, _translate("ApplianceWizard", "1"))
        self.uiSummaryTreeWidget.headerItem().setText(1, _translate("ApplianceWizard", "2"))
        __sortingEnabled = self.uiSummaryTreeWidget.isSortingEnabled()
        self.uiSummaryTreeWidget.setSortingEnabled(False)
        self.uiSummaryTreeWidget.topLevelItem(0).setText(0, _translate("ApplianceWizard", "adapter_type"))
        self.uiSummaryTreeWidget.topLevelItem(0).setText(1, _translate("ApplianceWizard", "e1000"))
        self.uiSummaryTreeWidget.topLevelItem(1).setText(0, _translate("ApplianceWizard", "console_type"))
        self.uiSummaryTreeWidget.topLevelItem(1).setText(1, _translate("ApplianceWizard", "telnet"))
        self.uiSummaryTreeWidget.topLevelItem(2).setText(0, _translate("ApplianceWizard", "ram"))
        self.uiSummaryTreeWidget.topLevelItem(2).setText(1, _translate("ApplianceWizard", "3072"))
        self.uiSummaryTreeWidget.topLevelItem(3).setText(0, _translate("ApplianceWizard", "arch"))
        self.uiSummaryTreeWidget.topLevelItem(3).setText(1, _translate("ApplianceWizard", "x68_64"))
        self.uiSummaryTreeWidget.topLevelItem(4).setText(0, _translate("ApplianceWizard", "adapters"))
        self.uiSummaryTreeWidget.topLevelItem(4).setText(1, _translate("ApplianceWizard", "16"))
        self.uiSummaryTreeWidget.topLevelItem(5).setText(0, _translate("ApplianceWizard", "kernel command line"))
        self.uiSummaryTreeWidget.topLevelItem(5).setText(1, _translate("ApplianceWizard", "user=gns3"))
        self.uiSummaryTreeWidget.setSortingEnabled(__sortingEnabled)
        self.uiUsageWizardPage.setTitle(_translate("ApplianceWizard", "Usage"))
        self.uiUsageWizardPage.setSubTitle(_translate("ApplianceWizard", "Please read the following instructions in order to use your new appliance."))
        self.uiUsageTextEdit.setHtml(_translate("ApplianceWizard", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'.SF NS Text\'; font-size:13pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Ubuntu\'; font-size:11pt;\">The default username/password is admin/admin. A default configuration is present.</span></p></body></html>"))

from . import resources_rc



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import json
import pytest
import binascii
import subprocess

from gns3.servers import Servers
from gns3.qt import QtWidgets
from unittest.mock import MagicMock, patch


@pytest.fixture(autouse=True)
def reset_server():
    Servers._instance = None


def test_loadSettings_EmptySettings(local_config, tmpdir):

    with open(str(tmpdir / "test.cfg"), "w+") as f:
        json.dump({
            "version": "1.4"
        }, f)

    local_config.setConfigFilePath(str(tmpdir / "test.cfg"))

    Servers._instance = None
    servers = Servers.instance()

    assert servers.localServerSettings()["port"] == 8000
    assert len(servers.localServerSettings()["password"]) == 64
    assert len(servers.localServerSettings()["user"]) == 64

    with open(str(tmpdir / "test.cfg")) as f:
        conf = json.load(f)
        assert servers.localServerSettings()["password"] == conf["Servers"]["local_server"]["password"]
        assert servers.localServerSettings()["user"] == conf["Servers"]["local_server"]["user"]


def test_loadSettings(tmpdir, local_config):
    with open(str(tmpdir / "test.cfg"), "w+") as f:
        json.dump({
            "Servers": {
                "local_server": {
                    "auth": True,
                    "user": "world",
                    "password": "hello"
                }
            },
            "version": "1.4"
        }, f)

    local_config.setConfigFilePath(str(tmpdir / "test.cfg"))
    Servers._instance = None
    servers = Servers.instance()

    assert servers.localServerSettings()["password"] == "hello"


def test_loadSettingsWith13LocalServerSetting(tmpdir, local_config):
    with open(str(tmpdir / "test.cfg"), "w+") as f:
        json.dump({
            "Servers": {
                "local_server": {
                    "auth": True,
                    "user": "world",
                    "password": "hello"
                }
            },
            "LocalServer": {
                "auth": False
            },
            "version": "1.4"
        }, f)

    local_config.setConfigFilePath(str(tmpdir / "test.cfg"))
    Servers._instance = None
    servers = Servers.instance()

    local_server = local_config.loadSectionSettings("LocalServer", {})

    assert local_server["auth"] == True
    assert local_server["user"] == "world"
    assert local_server["password"] == "hello"


def testServers():
    servers = Servers.instance()
    http_server = servers.getRemoteServer("http", "localhost", 8000, None)
    assert len(servers.servers()) == 2


def test_getRemoteServer():
    servers = Servers.instance()
    http_server = servers.getRemoteServer("http", "localhost", 8000, None)
    assert http_server.protocol() == "http"
    assert http_server.host() == "localhost"
    assert http_server.port() == 8000
    assert http_server.user() is None


def test_getServerFromString():

    servers = Servers.instance()
    server = servers.getServerFromString("127.0.0.1:4000")
    assert server.protocol() == "http"
    assert server.host() == "127.0.0.1"
    assert server.port() == 4000
    assert server.user() is None


def test_getServerFromString_with_user():

    servers = Servers.instance()
    server = servers.getServerFromString("http://root@127.0.0.1:4000")
    assert server.protocol() == "http"
    assert server.host() == "127.0.0.1"
    assert server.port() == 4000
    assert server.user() == "root"


def test_is_non_local_server_configured():

    servers = Servers.instance()

    assert servers.isNonLocalServerConfigured() is False
    servers._vm_server = object()
    assert servers.isNonLocalServerConfigured() is True
    servers._vm_server = None
    assert servers.isNonLocalServerConfigured() is False


def test_handle_handleSslErrors():
    """
    Simulate when user accept an insecure certificate. This should be save to the configuration
    """

    servers = Servers.instance()

    servers._addRemoteServer("https", "127.0.0.1", "443", user="root", password="toto")
    servers._saveSettings()
    assert servers._settings["remote_servers"] == [
        {
            'accept_insecure_certificate': None,
            'host': '127.0.0.1',
            'password': 'toto',
            'port': 443,
            'protocol': 'https',
            'ram_limit': 0,
            'url': 'https://root@127.0.0.1:443',
            'user': 'root'
        }
    ]

    reply = MagicMock()
    reply.url.return_value.toDisplayString.return_value = "https://root@127.0.0.1:443/v1/version"
    ssl_error = MagicMock()
    ssl_error.certificate.return_value.digest.return_value = binascii.unhexlify("cca0a932ced2fb1b1a18c823542cb065")
    errorList = [ssl_error]

    with patch("gns3.qt.QtWidgets.QMessageBox.warning") as message_box_mock:
        message_box_mock.return_value = QtWidgets.QMessageBox.Yes
        servers._handleSslErrors(reply, errorList)

    servers._saveSettings()
    assert len(servers._settings["remote_servers"]) == 1
    assert servers._settings["remote_servers"] == [
        {
            'accept_insecure_certificate': 'cca0a932ced2fb1b1a18c823542cb065',
            'host': '127.0.0.1',
            'password': 'toto',
            'port': 443,
            'protocol': 'https',
            'ram_limit': 0,
            'url': 'https://root@127.0.0.1:443',
            'user': 'root'
        }
    ]


@pytest.mark.skipif(sys.platform.startswith('win') is True, reason='Not for windows')
def test_startLocalServer(tmpdir, local_config):
    local_server_path = str(tmpdir / "gns3server")
    open(local_server_path, "w+").close()

    with open(str(tmpdir / "test.cfg"), "w+") as f:
        json.dump({
            "Servers": {
                "local_server": {
                    "path": local_server_path,
                }
            },
            "version": "1.4"
        }, f)

    local_config.setConfigFilePath(str(tmpdir / "test.cfg"))
    Servers._instance = None

    with patch("gns3.local_config.LocalConfig.configDirectory") as mock_local_config:
        mock_local_config.return_value = str(tmpdir)
        process_mock = MagicMock()
        with patch("subprocess.Popen", return_value=process_mock) as mock:

            # If everything work fine the command is still running and a timeout is raised
            process_mock.communicate.side_effect = subprocess.TimeoutExpired("test", 1)

            Servers.instance().startLocalServer()
            mock.assert_called_with([local_server_path,
                                     '--host=127.0.0.1',
                                     '--port=8000',
                                     '--local',
                                     '--debug',
                                     '--log='  + str(tmpdir / "gns3_server.log"),
                                     '--pid=' + str(tmpdir / "gns3_server.pid")
                                    ])


def test_killAlreadyRunningServer(tmpdir):
    with patch("gns3.local_config.LocalConfig.configDirectory") as mock_local_config:
        mock_local_config.return_value = str(tmpdir)

        with open(str(tmpdir / "gns3_server.pid"), "w+") as f:
            f.write("42")

        mock_process = MagicMock()
        with patch("psutil.Process", return_value=mock_process) as mock:
            Servers.instance()._killAlreadyRunningServer()
            mock.assert_called_with(pid=42)
            assert mock_process.kill.called




# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Built-in module implementation.
"""

from gns3.qt import QtWidgets
from ..module import Module
from .cloud import Cloud
from .host import Host


import logging
log = logging.getLogger(__name__)


class Builtin(Module):

    """
    Built-in module.
    """

    def __init__(self):
        super().__init__()

        self._nodes = []

    def configChangedSlot(self):
        pass

    def addNode(self, node):
        """
        Adds a node to this module.

        :param node: Node instance
        """

        self._nodes.append(node)

    def removeNode(self, node):
        """
        Removes a node from this module.

        :param node: Node instance
        """

        if node in self._nodes:
            self._nodes.remove(node)

    def reset(self):
        """
        Resets the module.
        """

        log.info("Built-in module reset")
        self._nodes.clear()

    def createNode(self, node_class, server, project):
        """
        Creates a new node.

        :param node_class: Node object
        :param server: HTTPClient instance
        :param project: Project instance
        """

        log.info("creating node {}".format(node_class))

        # create an instance of the node class
        return node_class(self, server, project)

    def setupNode(self, node, node_name):
        """
        Setups a node.

        :param node: Node instance
        :param node_name: Node name
        """

        log.info("configuring node {}".format(node))
        node.setup()

    @staticmethod
    def findAlternativeInterface(node, missing_interface):

        from gns3.main_window import MainWindow
        mainwindow = MainWindow.instance()

        available_interfaces = []
        for interface in node.settings()["interfaces"]:
            available_interfaces.append(interface["name"])

        if available_interfaces:
            selection, ok = QtWidgets.QInputDialog.getItem(mainwindow,
                                                           "Cloud interfaces", "Interface {} could not be found\nPlease select an alternative from your existing interfaces:".format(missing_interface),
                                                           available_interfaces, 0, False)
            if ok:
                return selection
            QtWidgets.QMessageBox.warning(mainwindow, "Cloud interface", "No alternative interface chosen to replace {} on this host, this may lead to issues".format(missing_interface))
            return None
        else:
            QtWidgets.QMessageBox.critical(mainwindow, "Cloud interface", "Could not find interface {} on this host".format(missing_interface))
            return missing_interface

    @staticmethod
    def getNodeClass(name):
        """
        Returns the object with the corresponding name.

        :param name: object name
        """

        if name in globals():
            return globals()[name]
        return None

    @staticmethod
    def classes():
        """
        Returns all the node classes supported by this module.

        :returns: list of classes
        """

        return [Cloud, Host]

    def nodes(self):
        """
        Returns all the node data necessary to represent a node
        in the nodes view and create a node on the scene.
        """

        nodes = []
        for node_class in Builtin.classes():
            nodes.append(
                {"class": node_class.__name__,
                 "name": node_class.symbolName(),
                 "categories": node_class.categories(),
                 "symbol": node_class.defaultSymbol(),
                 "builtin": True}
            )
        return nodes

    @staticmethod
    def preferencePages():
        """
        :returns: QWidget object list
        """

        return []

    @staticmethod
    def instance():
        """
        Singleton to return only on instance of built-in module.

        :returns: instance of Builtin
        """

        if not hasattr(Builtin, "_instance"):
            Builtin._instance = Builtin()
        return Builtin._instance



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import os
import hashlib
import shutil
import tarfile

import logging
log = logging.getLogger(__name__)


class Image:
    """
    An appliance image file.
    """

    # Cache md5sum in order to improve performances
    _cache = {}

    def __init__(self, path):
        """
        :params: path of the image
        """

        self.path = path
        self._md5sum = None
        self._version = None

    @property
    def filename(self):
        """
        :returns: Image filename
        """
        return os.path.basename(self.path)

    @property
    def version(self):
        """
        :returns: Get the file version / release
        """
        return self._version

    @version.setter
    def version(self, version):
        """
        :returns: Set the file version / release
        """
        self._version = version

    @property
    def md5sum(self):
        """
        Compute a md5 hash for file

        :params cache: Cache sum on disk
        :returns: hexadecimal md5
        """

        if self._md5sum is None:
            from_cache = Image._cache.get(self.path)
            if from_cache:
                self._md5sum = from_cache
                return self._md5sum

            if os.path.exists(self.path + ".md5sum"):
                with open(self.path + ".md5sum", encoding="utf-8") as f:
                    self._md5sum = f.read()
                    return self._md5sum

            if not os.path.isfile(self.path):
                return None
            m = hashlib.md5()
            with open(self.path, "rb") as f:
                while True:
                    buf = f.read(4096)
                    if not buf:
                        break
                    m.update(buf)
            self._md5sum = m.hexdigest()
        Image._cache[self.path] = self._md5sum
        return self._md5sum

    @property
    def filesize(self):
        """
        Return image file size
        """
        try:
            return os.path.getsize(self.path)
        except OSError:
            return 0

    def copy(self, directory, filename):
        """
        Copy the image to a directory. Extract the image if it's an OVA.

        The destination directory is created if not exists

        :param directory: Destination directory
        """
        log.debug("Copy %s to %s", directory, filename)

        is_tar = False
        if tarfile.is_tarfile(self.path):
            if '/' in filename or '\\' in filename:
                # In case of OVA we want to update the OVA name
                base_file = re.split(r'[/\\]', filename)[0]
            else:
                base_file = filename
            dst = os.path.join(directory, base_file)

            # is_tarfile can have false positive if file start with 00000 like ISO
            # we check if we have file in the tar
            tar = tarfile.open(self.path)
            if len(tar.getnames()) > 0:
                is_tar = True
                os.makedirs(dst, exist_ok=True)
                tar.extractall(path=dst)
            tar.close()

        if not is_tar:
            dst = os.path.join(directory, filename)
            os.makedirs(directory, exist_ok=True)
            shutil.copy(self.path, dst)
        with open(dst + ".md5sum", "w+", encoding="utf-8") as f:
            f.write(self.md5sum)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/noplay/code/gns3/gns3-gui/gns3/ui/iouvm_converter_wizard.ui'
#
# Created by: PyQt5 UI code generator 5.5
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_IOUVMConverterWizard(object):

    def setupUi(self, IOUVMConverterWizard):
        IOUVMConverterWizard.setObjectName("IOUVMConverterWizard")
        IOUVMConverterWizard.resize(649, 377)
        IOUVMConverterWizard.setOptions(QtWidgets.QWizard.NoCancelButton | QtWidgets.QWizard.NoDefaultButton)
        self.uiWizardWelcomePage = QtWidgets.QWizardPage()
        self.uiWizardWelcomePage.setObjectName("uiWizardWelcomePage")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.uiWizardWelcomePage)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label = QtWidgets.QLabel(self.uiWizardWelcomePage)
        self.label.setWordWrap(True)
        self.label.setObjectName("label")
        self.verticalLayout_2.addWidget(self.label)
        IOUVMConverterWizard.addPage(self.uiWizardWelcomePage)
        self.uiWizardStartConfigureGNS3 = QtWidgets.QWizardPage()
        self.uiWizardStartConfigureGNS3.setObjectName("uiWizardStartConfigureGNS3")
        self.verticalLayout_13 = QtWidgets.QVBoxLayout(self.uiWizardStartConfigureGNS3)
        self.verticalLayout_13.setObjectName("verticalLayout_13")
        self.label_14 = QtWidgets.QLabel(self.uiWizardStartConfigureGNS3)
        self.label_14.setObjectName("label_14")
        self.verticalLayout_13.addWidget(self.label_14)
        IOUVMConverterWizard.addPage(self.uiWizardStartConfigureGNS3)
        self.uiWizardVirtualBoxNatPage = QtWidgets.QWizardPage()
        self.uiWizardVirtualBoxNatPage.setObjectName("uiWizardVirtualBoxNatPage")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.uiWizardVirtualBoxNatPage)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.label_2 = QtWidgets.QLabel(self.uiWizardVirtualBoxNatPage)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_3.addWidget(self.label_2)
        IOUVMConverterWizard.addPage(self.uiWizardVirtualBoxNatPage)
        self.uiWizardUpdateIOUVM = QtWidgets.QWizardPage()
        self.uiWizardUpdateIOUVM.setObjectName("uiWizardUpdateIOUVM")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.uiWizardUpdateIOUVM)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.label_3 = QtWidgets.QLabel(self.uiWizardUpdateIOUVM)
        self.label_3.setTextFormat(QtCore.Qt.RichText)
        self.label_3.setAlignment(QtCore.Qt.AlignLeading | QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_4.addWidget(self.label_3)
        IOUVMConverterWizard.addPage(self.uiWizardUpdateIOUVM)
        self.uiWizardPageVirtualBoxHostOnly = QtWidgets.QWizardPage()
        self.uiWizardPageVirtualBoxHostOnly.setObjectName("uiWizardPageVirtualBoxHostOnly")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.uiWizardPageVirtualBoxHostOnly)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.label_4 = QtWidgets.QLabel(self.uiWizardPageVirtualBoxHostOnly)
        self.label_4.setObjectName("label_4")
        self.verticalLayout_5.addWidget(self.label_4)
        IOUVMConverterWizard.addPage(self.uiWizardPageVirtualBoxHostOnly)
        self.uiWizardDownloadBackups = QtWidgets.QWizardPage()
        self.uiWizardDownloadBackups.setObjectName("uiWizardDownloadBackups")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.uiWizardDownloadBackups)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.label_10 = QtWidgets.QLabel(self.uiWizardDownloadBackups)
        self.label_10.setObjectName("label_10")
        self.verticalLayout_10.addWidget(self.label_10)
        IOUVMConverterWizard.addPage(self.uiWizardDownloadBackups)
        self.uiWizardRestoreBackups = QtWidgets.QWizardPage()
        self.uiWizardRestoreBackups.setObjectName("uiWizardRestoreBackups")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout(self.uiWizardRestoreBackups)
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.label_11 = QtWidgets.QLabel(self.uiWizardRestoreBackups)
        self.label_11.setWordWrap(True)
        self.label_11.setObjectName("label_11")
        self.verticalLayout_11.addWidget(self.label_11)
        IOUVMConverterWizard.addPage(self.uiWizardRestoreBackups)
        self.uiWizardPageBackup = QtWidgets.QWizardPage()
        self.uiWizardPageBackup.setObjectName("uiWizardPageBackup")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.uiWizardPageBackup)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.label_5 = QtWidgets.QLabel(self.uiWizardPageBackup)
        font = QtGui.QFont()
        font.setPointSize(43)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.verticalLayout_6.addWidget(self.label_5)
        IOUVMConverterWizard.addPage(self.uiWizardPageBackup)
        self.uiWizardPageIOURCCheck = QtWidgets.QWizardPage()
        self.uiWizardPageIOURCCheck.setObjectName("uiWizardPageIOURCCheck")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.uiWizardPageIOURCCheck)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label_7 = QtWidgets.QLabel(self.uiWizardPageIOURCCheck)
        self.label_7.setWordWrap(True)
        self.label_7.setObjectName("label_7")
        self.verticalLayout.addWidget(self.label_7)
        IOUVMConverterWizard.addPage(self.uiWizardPageIOURCCheck)
        self.uiWizardUpdateConfiguration = QtWidgets.QWizardPage()
        self.uiWizardUpdateConfiguration.setObjectName("uiWizardUpdateConfiguration")
        self.verticalLayout_12 = QtWidgets.QVBoxLayout(self.uiWizardUpdateConfiguration)
        self.verticalLayout_12.setObjectName("verticalLayout_12")
        self.label_12 = QtWidgets.QLabel(self.uiWizardUpdateConfiguration)
        self.label_12.setWordWrap(True)
        self.label_12.setObjectName("label_12")
        self.verticalLayout_12.addWidget(self.label_12)
        IOUVMConverterWizard.addPage(self.uiWizardUpdateConfiguration)
        self.uiWizardPagePatchTopologies = QtWidgets.QWizardPage()
        self.uiWizardPagePatchTopologies.setObjectName("uiWizardPagePatchTopologies")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.uiWizardPagePatchTopologies)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.label_6 = QtWidgets.QLabel(self.uiWizardPagePatchTopologies)
        self.label_6.setWordWrap(True)
        self.label_6.setObjectName("label_6")
        self.verticalLayout_7.addWidget(self.label_6)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.uiLineEditTopologiesPath = QtWidgets.QLineEdit(self.uiWizardPagePatchTopologies)
        self.uiLineEditTopologiesPath.setObjectName("uiLineEditTopologiesPath")
        self.horizontalLayout_2.addWidget(self.uiLineEditTopologiesPath)
        self.uiPushButtonBrowse = QtWidgets.QPushButton(self.uiWizardPagePatchTopologies)
        self.uiPushButtonBrowse.setObjectName("uiPushButtonBrowse")
        self.horizontalLayout_2.addWidget(self.uiPushButtonBrowse)
        self.verticalLayout_7.addLayout(self.horizontalLayout_2)
        IOUVMConverterWizard.addPage(self.uiWizardPagePatchTopologies)
        self.uiWizardCongratulation = QtWidgets.QWizardPage()
        self.uiWizardCongratulation.setObjectName("uiWizardCongratulation")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.uiWizardCongratulation)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.label_8 = QtWidgets.QLabel(self.uiWizardCongratulation)
        self.label_8.setWordWrap(True)
        self.label_8.setObjectName("label_8")
        self.verticalLayout_8.addWidget(self.label_8)
        IOUVMConverterWizard.addPage(self.uiWizardCongratulation)

        self.retranslateUi(IOUVMConverterWizard)
        QtCore.QMetaObject.connectSlotsByName(IOUVMConverterWizard)

    def retranslateUi(self, IOUVMConverterWizard):
        _translate = QtCore.QCoreApplication.translate
        IOUVMConverterWizard.setWindowTitle(_translate("IOUVMConverterWizard", "GNS3 IOU Converter"))
        self.label.setText(_translate("IOUVMConverterWizard", "<h1>Welcome to the IOUVM converter</h1>\n"
                                      "<p>This Wizard will help you to convert the IOUVM to the new GNS3 VM</p>\n"
                                      "<p>The GNS3 VM has a self update system allowing you to easily upgrade between version of the GNS3 VM without manual operations.</p>"))
        self.label_14.setText(_translate("IOUVMConverterWizard", "<h1>Start GNS3</h1>\n"
                                         "<ul>\n"
                                         "<li>Start GNS3 >= 1.4</li>\n"
                                         "<li>Configure the GNS3 VM via preferences or the initial wizard</li>\n"
                                         "<li>Exit all instance of GNS3</li>\n"
                                         "</ul>"))
        self.label_2.setText(_translate("IOUVMConverterWizard", "<h1>Setup NAT</h1>\n"
                                        "<ul>\n"
                                        "<li>Start VirtualBox </li>\n"
                                        "<li>Click on the IOUVM</li>\n"
                                        "<li>Click on settings</li>\n"
                                        "<li>Click on Network</li>\n"
                                        "<li>Change Host-only adapter to NAT</li>\n"
                                        "<li>Click on OK</li>\n"
                                        "<li>Start the VM<li>\n"
                                        "</ul>"))
        self.label_3.setText(_translate("IOUVMConverterWizard", "<html><head/><body><p><span style=\" font-size:xx-large; font-weight:600;\">Update GNS3 on IOUVM</span></p><p>In the VM: </p><ul style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" margin-top:12px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Log as root with password CISCO</li><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">pip3 install gns3-server==1.3.10</li><li style=\" margin-top:0px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">halt</li></ul></body></html>"))
        self.label_4.setText(_translate("IOUVMConverterWizard", "<h1>Restore VM network settings</h1>\n"
                                        "<ul>\n"
                                        "<li>Start VirtualBox </li>\n"
                                        "<li>Click on the IOUVM</li>\n"
                                        "<li>Click on settings</li>\n"
                                        "<li>Click on Network</li>\n"
                                        "<li>Change NAT adapter to Host-Only</li>\n"
                                        "<li>Click on OK</li>\n"
                                        "<li>Start the VM<li>\n"
                                        "</ul>"))
        self.label_10.setText(_translate("IOUVMConverterWizard", "<h1>Download backups</h1>\n"
                                         "<ul>\n"
                                         "<li>Log as root with password cisco</li>\n"
                                         "<li>Type ifconfig eth0</li>\n"
                                         "<li>Remember the addr</li>\n"
                                         "<li>In your browser open http://THEADDR:8000</li>\n"
                                         "<li>Download projects and images backups</li>\n"
                                         "<li>halt the vm</li>\n"
                                         "</ul>"))
        self.label_11.setText(_translate("IOUVMConverterWizard", "<h1>Restore backups</h1>\n"
                                         "<ul>\n"
                                         "<li>Start the GNS3 VM</li>\n"
                                         "<li>Remember the IP display on the information screen</li>\n"
                                         "<li>In a browser open http://THEIP:8000/upload</li>\n"
                                         "<li>Select the option for restoring projects and upload the previous backup</li>\n"
                                         "<li>Select the option for restoring images and upload the previous backup</li>\n"
                                         "</ul>"))
        self.label_5.setText(_translate("IOUVMConverterWizard", "<center>\n"
                                        "BACKUP<br>\n"
                                        "YOUR TOPOLOGIES!\n"
                                        "</center>"))
        self.label_7.setText(_translate("IOUVMConverterWizard", "<h1>Validation of the IOURC file</h1> \n"
                                        "The IOURC file contain your licence for IOU.<br>\n"
                                        "The format of file is:\n"
                                        "<pre>\n"
                                        "[license]\n"
                                        "gns3-iouvm = OLDLICENCE;\n"
                                        "gns3vm = NEWLICENCE;\n"
                                        "</pre>\n"
                                        "<br>\n"
                                        "The GNS3VM require a new licence. You need to add it in the file. IOU is a CISCO product. GNS3 staff is not allowed to provide this licence."))
        self.label_12.setText(_translate("IOUVMConverterWizard", "<h1>Update configuration</h1>\n"
                                         "<p style=\"color: red\">Your GNS3 configuration will be updated.</p>\n"
                                         "<p>All your remote servers will be removed and replace by the GNS3VM. </p><p>If you have a custom remote server <strong>DO NOT CONTINUE</strong></p>"))
        self.label_6.setText(_translate("IOUVMConverterWizard", "<h1>Patch topologies</h1>\n"
                                        "We need to patch your topologies please select your topologies directory"))
        self.uiPushButtonBrowse.setText(_translate("IOUVMConverterWizard", "Browse..."))
        self.label_8.setText(_translate("IOUVMConverterWizard", "<center>\n"
                                        "<h1>Congratulation you can now use the GNS3 VM</h1>\n"
                                        "</center>"))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from unittest.mock import patch
from gns3.modules.dynamips import Dynamips


def test_getDefaultIdlePC(tmpdir):

    fake_img = str(tmpdir / 'fake')
    open(fake_img, 'w+').close()

    # Unknow image
    assert Dynamips.getDefaultIdlePC(fake_img) is None

    # Know image with full path
    with patch('gns3.modules.dynamips.Dynamips._md5sum', return_value='7f4ae12a098391bc0edcaf4f44caaf9d'):
        assert Dynamips.getDefaultIdlePC(fake_img) == '0x80358a60'

    # Know image with relative path
    with patch('gns3.image_manager.ImageManager.getDirectoryForType', return_value=str(tmpdir)):
        with patch('gns3.modules.dynamips.Dynamips._md5sum', return_value='7f4ae12a098391bc0edcaf4f44caaf9d'):
            assert Dynamips.getDefaultIdlePC('fake') == '0x80358a60'



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pathlib
import pytest
import os
from unittest.mock import patch

from gns3.image_manager import ImageManager

@pytest.fixture
def images_dir(tmpdir):
    path = tmpdir / "images"
    path.mkdir()
    return path

@pytest.yield_fixture
def image_manager(tmpdir, images_dir):
    ImageManager._instance = None
    with patch('gns3.servers.Servers.localServerSettings', return_value={'images_path': str(images_dir)}):
        yield ImageManager.instance()


@pytest.fixture
def qemu_img(images_dir):
    """
    Return a fake qemu IMG
    """
    path = str(images_dir / 'QEMU' / 'test.img')
    os.makedirs(str(images_dir / 'QEMU'))
    open(path, 'w+').close()
    return path


def test_askCopyUploadImage_remote(image_manager, remote_server):
    with patch('gns3.image_manager.ImageManager._uploadImageToRemoteServer') as mock:
        image_manager.askCopyUploadImage(None, '/tmp/test', remote_server, 'QEMU')
        assert mock.called


def test_uploadImageToRemoteServer(image_manager, remote_server, images_dir):
    with patch('gns3.http_client.HTTPClient.post') as mock:
        filename = image_manager._uploadImageToRemoteServer(str(images_dir / "QEMU" / "test"), remote_server, 'QEMU')
        assert filename == 'test'
        args, kwargs = mock.call_args
        assert args[0] == '/qemu/vms/test'
        assert kwargs['body'] == pathlib.Path(str(images_dir / "QEMU" / "test"))


def test_getDirectory(image_manager, images_dir):
    assert image_manager.getDirectory() == str(images_dir)


def test_directoryType(image_manager, images_dir):
    assert image_manager.getDirectoryForType('DYNAMIPS') == str(images_dir / 'IOS')
    assert image_manager.getDirectoryForType('QEMU') == str(images_dir / 'QEMU')
    assert image_manager.getDirectoryForType('IOU') == str(images_dir / 'IOU')


def test_addMissingImage(image_manager, remote_server, qemu_img):
    with patch('gns3.image_manager.ImageManager._uploadImageToRemoteServer') as mock:
        with patch('gns3.image_manager.ImageManager._askForUploadMissingImage', return_value=True):
            image_manager.addMissingImage('test.img', remote_server, 'QEMU')
            assert mock.called
            args, kwargs = mock.call_args
            assert args[0] == qemu_img
            assert args[1] == remote_server
            assert args[2] == 'QEMU'


def test_addMissingImageOVAWithMultipleVMDK(image_manager, remote_server, images_dir):
    os.makedirs(str(images_dir / 'QEMU' / 'test.ova'))
    open(str(images_dir / 'QEMU' / 'test.ova' / 'test.vmdk'), 'w+').close()
    open(str(images_dir / 'QEMU' / 'test.ova' / 'test-s001.vmdk'), 'w+').close()
    open(str(images_dir / 'QEMU' / 'test.ova' / 'test.nvram'), 'w+').close()

    with patch('gns3.image_manager.ImageManager._uploadImageToRemoteServer') as mock:
        with patch('gns3.image_manager.ImageManager._askForUploadMissingImage', return_value=True):
            image_manager.addMissingImage('test.ova/test.vmdk', remote_server, 'QEMU')
            assert mock.call_count == 2

            args, kwargs = mock.call_args_list[0]
            assert args[0] == str(images_dir / 'QEMU' / 'test.ova' / 'test-s001.vmdk')
            assert args[1] == remote_server
            assert args[2] == 'QEMU'

            args, kwargs = mock.call_args_list[1]
            assert os.path.normpath(args[0]) == str(images_dir / 'QEMU' / 'test.ova' / 'test.vmdk')
            assert args[1] == remote_server
            assert args[2] == 'QEMU'


def test_addMissingImage_ask_once(image_manager, remote_server, qemu_img):
    with patch('gns3.image_manager.ImageManager._askForUploadMissingImage', return_value=False) as mock:
        image_manager.addMissingImage('test.img', remote_server, 'QEMU')
        image_manager.addMissingImage('test.img', remote_server, 'QEMU')
        assert mock.call_count == 1


def test_addMissingImageLocalServer(image_manager, local_server, qemu_img):
    with patch('gns3.image_manager.ImageManager._uploadImageToRemoteServer') as mock:
        image_manager.addMissingImage('test.img', local_server, 'QEMU')
        assert not mock.called


def test_getRelativeImagePath(image_manager, remote_server, qemu_img):
    assert image_manager._getRelativeImagePath(qemu_img, "QEMU") == "test.img"


def test_getRelativeImagePathOutsideStandardDirectory(image_manager, remote_server, images_dir):
    qemu_img_abs = images_dir / "full_images" / "a.img"
    qemu_img_abs.write("1", ensure=True)

    assert image_manager._getRelativeImagePath(str(qemu_img_abs), "QEMU") == "a.img"




# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Dialog to load module and built-in preference pages.
"""

from ..qt import QtCore, QtWidgets
from ..ui.preferences_dialog_ui import Ui_PreferencesDialog
from ..pages.server_preferences_page import ServerPreferencesPage
from ..pages.general_preferences_page import GeneralPreferencesPage
from ..pages.packet_capture_preferences_page import PacketCapturePreferencesPage
from ..modules import MODULES


class PreferencesDialog(QtWidgets.QDialog, Ui_PreferencesDialog):

    """
    Preferences dialog implementation.

    :param parent: parent widget
    """

    def __init__(self, parent):

        super().__init__(parent)

        self.setupUi(self)

        # We adapt the max size to the screen resolution
        # We need to manually do that otherwise on small screen the windows
        # could be bigger than the screen instead of displaying scrollbars
        height = QtWidgets.QDesktopWidget().screenGeometry().height() - 100
        width = QtWidgets.QDesktopWidget().screenGeometry().width() - 100

        self.setMaximumSize(QtCore.QSize(width, height))
        if width > 900 and self.width() < 900:
            self.resize(900, self.height())
        if height > 768 and self.height() < 768:
            self.resize(self.width(), 768)

        self.uiTreeWidget.currentItemChanged.connect(self._showPreferencesPageSlot)
        self._applyButton = self.uiButtonBox.button(QtWidgets.QDialogButtonBox.Apply)
        self._applyButton.clicked.connect(self._applyPreferences)
        self._applyButton.setEnabled(False)
        self._applyButton.setStyleSheet("QPushButton:disabled {color: gray}")
        self._items = []
        self._loadPreferencePages()

        # select the first available page
        self.uiTreeWidget.setCurrentItem(self._items[0])

        # Something has change?
        self._modified = False



    def _loadPreferencePages(self):
        """
        Loads all preference pages (built-ins and from modules).
        """

        # load built-in preference pages
        pages = [
            GeneralPreferencesPage,
            ServerPreferencesPage,
            PacketCapturePreferencesPage,
        ]

        for page in pages:
            preferences_page = page(self)
            preferences_page.loadPreferences()
            name = preferences_page.windowTitle()
            item = QtWidgets.QTreeWidgetItem(self.uiTreeWidget)
            item.setText(0, name)
            item.setData(0, QtCore.Qt.UserRole, preferences_page)
            self.uiStackedWidget.addWidget(preferences_page)
            self._items.append(item)
            self._watchForChanges(preferences_page)

        # load module preference pages
        for module in MODULES:
            preference_pages = module.preferencePages()
            parent = self.uiTreeWidget
            for cls in preference_pages:
                preferences_page = cls()
                preferences_page.loadPreferences()
                name = preferences_page.windowTitle()
                item = QtWidgets.QTreeWidgetItem(parent)
                item.setText(0, name)
                item.setData(0, QtCore.Qt.UserRole, preferences_page)
                self.uiStackedWidget.addWidget(preferences_page)
                self._items.append(item)
                if cls is preference_pages[0]:
                    parent = item
                self._watchForChanges(preferences_page)

        # expand all items by default
        self.uiTreeWidget.expandAll()

    def _watchForChanges(self, preferences_page):
        """
        Connect all the widget of a page to check if something has change
        """

        # Class name, changed signal
        widget_to_watch = {
            QtWidgets.QLineEdit: "textChanged",
            QtWidgets.QTreeWidget: "itemChanged",
            QtWidgets.QComboBox: "currentIndexChanged",
            QtWidgets.QSpinBox: "valueChanged",
            QtWidgets.QAbstractButton: "pressed"
        }

        for widget, signal in widget_to_watch.items():
            for children in preferences_page.findChildren(widget):
                getattr(children, signal).connect(self._preferenceChangeSlot)

    def _preferenceChangeSlot(self, *args):
        """
        Called when somthing change in the preference dialog
        """
        self._applyButton.setEnabled(True)
        self._modified = True

    def _showPreferencesPageSlot(self, current, previous):
        """
        Shows a preference page in the current dialog.

        :param current: current preference page widget
        :param previous: ignored
        """

        if current is None:
            current = previous

        preferences_page = current.data(0, QtCore.Qt.UserRole)
        accessible_name = preferences_page.accessibleName()
        if accessible_name:
            self.uiTitleLabel.setText(accessible_name)
        else:
            name = preferences_page.windowTitle()
            self.uiTitleLabel.setText("{} preferences".format(name))
        index = self.uiStackedWidget.indexOf(preferences_page)
        widget = self.uiStackedWidget.widget(index)
        self.uiStackedWidget.setMinimumSize(widget.size())
        self.uiStackedWidget.resize(widget.size())
        self.uiStackedWidget.setCurrentIndex(index)

    def _applyPreferences(self):
        """
        Saves all the preferences.
        """

        success = True
        for item in self._items:
            preferences_page = item.data(0, QtCore.Qt.UserRole)
            ok = preferences_page.savePreferences()
            # if page.savePreferences() returns None, assume success
            if ok is not None and not ok:
                success = False
        if success:
            self._applyButton.setEnabled(False)
            self._modified = False
        return success

    def reject(self):
        """
        Closes this dialog.
        """

        if self._modified:
            reply = QtWidgets.QMessageBox.warning(self,
                                                  "Preferences",
                                                  "You have unsaved preferences.\n\nContinue without saving?",
                                                  QtWidgets.QMessageBox.Yes,
                                                  QtWidgets.QMessageBox.No)
            if reply == QtWidgets.QMessageBox.No:
                return
        QtWidgets.QDialog.reject(self)

    def accept(self):
        """
        Saves the preferences and closes this dialog.
        """

        # close the nodes dock to refresh the node list
        from ..main_window import MainWindow
        main_window = MainWindow.instance()
        main_window.uiNodesDockWidget.setVisible(False)
        main_window.uiNodesDockWidget.setWindowTitle("")

        if self._applyPreferences():
            QtWidgets.QDialog.accept(self)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/iou/ui/iou_device_configuration_page.ui'
#
# Created: Thu Feb  4 21:08:13 2016
#      by: PyQt5 UI code generator 5.2.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_iouDeviceConfigPageWidget(object):
    def setupUi(self, iouDeviceConfigPageWidget):
        iouDeviceConfigPageWidget.setObjectName("iouDeviceConfigPageWidget")
        iouDeviceConfigPageWidget.resize(569, 564)
        self.verticalLayout = QtWidgets.QVBoxLayout(iouDeviceConfigPageWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiTabWidget = QtWidgets.QTabWidget(iouDeviceConfigPageWidget)
        self.uiTabWidget.setObjectName("uiTabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.tab)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.uiGeneralgroupBox = QtWidgets.QGroupBox(self.tab)
        self.uiGeneralgroupBox.setStyleSheet("")
        self.uiGeneralgroupBox.setObjectName("uiGeneralgroupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.uiGeneralgroupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.uiNameLabel = QtWidgets.QLabel(self.uiGeneralgroupBox)
        self.uiNameLabel.setObjectName("uiNameLabel")
        self.gridLayout.addWidget(self.uiNameLabel, 0, 0, 1, 1)
        self.uiNameLineEdit = QtWidgets.QLineEdit(self.uiGeneralgroupBox)
        self.uiNameLineEdit.setObjectName("uiNameLineEdit")
        self.gridLayout.addWidget(self.uiNameLineEdit, 0, 1, 1, 1)
        self.uiSymbolLabel = QtWidgets.QLabel(self.uiGeneralgroupBox)
        self.uiSymbolLabel.setObjectName("uiSymbolLabel")
        self.gridLayout.addWidget(self.uiSymbolLabel, 2, 0, 1, 1)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.uiSymbolLineEdit = QtWidgets.QLineEdit(self.uiGeneralgroupBox)
        self.uiSymbolLineEdit.setObjectName("uiSymbolLineEdit")
        self.horizontalLayout_7.addWidget(self.uiSymbolLineEdit)
        self.uiSymbolToolButton = QtWidgets.QToolButton(self.uiGeneralgroupBox)
        self.uiSymbolToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiSymbolToolButton.setObjectName("uiSymbolToolButton")
        self.horizontalLayout_7.addWidget(self.uiSymbolToolButton)
        self.gridLayout.addLayout(self.horizontalLayout_7, 2, 1, 1, 1)
        self.uiCategoryLabel = QtWidgets.QLabel(self.uiGeneralgroupBox)
        self.uiCategoryLabel.setObjectName("uiCategoryLabel")
        self.gridLayout.addWidget(self.uiCategoryLabel, 3, 0, 1, 1)
        self.uiCategoryComboBox = QtWidgets.QComboBox(self.uiGeneralgroupBox)
        self.uiCategoryComboBox.setObjectName("uiCategoryComboBox")
        self.gridLayout.addWidget(self.uiCategoryComboBox, 3, 1, 1, 1)
        self.uiIOUImageLabel = QtWidgets.QLabel(self.uiGeneralgroupBox)
        self.uiIOUImageLabel.setObjectName("uiIOUImageLabel")
        self.gridLayout.addWidget(self.uiIOUImageLabel, 5, 0, 1, 1)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.uiIOUImageLineEdit = QtWidgets.QLineEdit(self.uiGeneralgroupBox)
        self.uiIOUImageLineEdit.setObjectName("uiIOUImageLineEdit")
        self.horizontalLayout_5.addWidget(self.uiIOUImageLineEdit)
        self.uiIOUImageToolButton = QtWidgets.QToolButton(self.uiGeneralgroupBox)
        self.uiIOUImageToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiIOUImageToolButton.setObjectName("uiIOUImageToolButton")
        self.horizontalLayout_5.addWidget(self.uiIOUImageToolButton)
        self.gridLayout.addLayout(self.horizontalLayout_5, 5, 1, 1, 1)
        self.uiStartupConfigLabel = QtWidgets.QLabel(self.uiGeneralgroupBox)
        self.uiStartupConfigLabel.setObjectName("uiStartupConfigLabel")
        self.gridLayout.addWidget(self.uiStartupConfigLabel, 6, 0, 1, 1)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.uiStartupConfigLineEdit = QtWidgets.QLineEdit(self.uiGeneralgroupBox)
        self.uiStartupConfigLineEdit.setObjectName("uiStartupConfigLineEdit")
        self.horizontalLayout_4.addWidget(self.uiStartupConfigLineEdit)
        self.uiStartupConfigToolButton = QtWidgets.QToolButton(self.uiGeneralgroupBox)
        self.uiStartupConfigToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiStartupConfigToolButton.setObjectName("uiStartupConfigToolButton")
        self.horizontalLayout_4.addWidget(self.uiStartupConfigToolButton)
        self.gridLayout.addLayout(self.horizontalLayout_4, 6, 1, 1, 1)
        self.uiPrivateConfigLabel = QtWidgets.QLabel(self.uiGeneralgroupBox)
        self.uiPrivateConfigLabel.setObjectName("uiPrivateConfigLabel")
        self.gridLayout.addWidget(self.uiPrivateConfigLabel, 7, 0, 1, 1)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.uiPrivateConfigLineEdit = QtWidgets.QLineEdit(self.uiGeneralgroupBox)
        self.uiPrivateConfigLineEdit.setObjectName("uiPrivateConfigLineEdit")
        self.horizontalLayout_6.addWidget(self.uiPrivateConfigLineEdit)
        self.uiPrivateConfigToolButton = QtWidgets.QToolButton(self.uiGeneralgroupBox)
        self.uiPrivateConfigToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiPrivateConfigToolButton.setObjectName("uiPrivateConfigToolButton")
        self.horizontalLayout_6.addWidget(self.uiPrivateConfigToolButton)
        self.gridLayout.addLayout(self.horizontalLayout_6, 7, 1, 1, 1)
        self.uiConsolePortLabel = QtWidgets.QLabel(self.uiGeneralgroupBox)
        self.uiConsolePortLabel.setObjectName("uiConsolePortLabel")
        self.gridLayout.addWidget(self.uiConsolePortLabel, 8, 0, 1, 1)
        self.uiConsolePortSpinBox = QtWidgets.QSpinBox(self.uiGeneralgroupBox)
        self.uiConsolePortSpinBox.setMaximum(65535)
        self.uiConsolePortSpinBox.setObjectName("uiConsolePortSpinBox")
        self.gridLayout.addWidget(self.uiConsolePortSpinBox, 8, 1, 1, 1)
        self.uiDefaultNameFormatLabel = QtWidgets.QLabel(self.uiGeneralgroupBox)
        self.uiDefaultNameFormatLabel.setObjectName("uiDefaultNameFormatLabel")
        self.gridLayout.addWidget(self.uiDefaultNameFormatLabel, 1, 0, 1, 1)
        self.uiDefaultNameFormatLineEdit = QtWidgets.QLineEdit(self.uiGeneralgroupBox)
        self.uiDefaultNameFormatLineEdit.setText("")
        self.uiDefaultNameFormatLineEdit.setObjectName("uiDefaultNameFormatLineEdit")
        self.gridLayout.addWidget(self.uiDefaultNameFormatLineEdit, 1, 1, 1, 1)
        self.verticalLayout_2.addWidget(self.uiGeneralgroupBox)
        self.uiOtherSettingsGroupBox = QtWidgets.QGroupBox(self.tab)
        self.uiOtherSettingsGroupBox.setObjectName("uiOtherSettingsGroupBox")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.uiOtherSettingsGroupBox)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.uiL1KeepalivesCheckBox = QtWidgets.QCheckBox(self.uiOtherSettingsGroupBox)
        self.uiL1KeepalivesCheckBox.setEnabled(True)
        self.uiL1KeepalivesCheckBox.setChecked(False)
        self.uiL1KeepalivesCheckBox.setObjectName("uiL1KeepalivesCheckBox")
        self.gridLayout_2.addWidget(self.uiL1KeepalivesCheckBox, 0, 0, 1, 2)
        self.uiDefaultValuesCheckBox = QtWidgets.QCheckBox(self.uiOtherSettingsGroupBox)
        self.uiDefaultValuesCheckBox.setChecked(True)
        self.uiDefaultValuesCheckBox.setObjectName("uiDefaultValuesCheckBox")
        self.gridLayout_2.addWidget(self.uiDefaultValuesCheckBox, 1, 0, 1, 2)
        self.uiRamLabel = QtWidgets.QLabel(self.uiOtherSettingsGroupBox)
        self.uiRamLabel.setObjectName("uiRamLabel")
        self.gridLayout_2.addWidget(self.uiRamLabel, 2, 0, 1, 1)
        self.uiRamSpinBox = QtWidgets.QSpinBox(self.uiOtherSettingsGroupBox)
        self.uiRamSpinBox.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRamSpinBox.sizePolicy().hasHeightForWidth())
        self.uiRamSpinBox.setSizePolicy(sizePolicy)
        self.uiRamSpinBox.setMinimum(32)
        self.uiRamSpinBox.setMaximum(65535)
        self.uiRamSpinBox.setSingleStep(32)
        self.uiRamSpinBox.setProperty("value", 128)
        self.uiRamSpinBox.setObjectName("uiRamSpinBox")
        self.gridLayout_2.addWidget(self.uiRamSpinBox, 2, 1, 1, 1)
        self.uiNvramLabel = QtWidgets.QLabel(self.uiOtherSettingsGroupBox)
        self.uiNvramLabel.setObjectName("uiNvramLabel")
        self.gridLayout_2.addWidget(self.uiNvramLabel, 3, 0, 1, 1)
        self.uiNvramSpinBox = QtWidgets.QSpinBox(self.uiOtherSettingsGroupBox)
        self.uiNvramSpinBox.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiNvramSpinBox.sizePolicy().hasHeightForWidth())
        self.uiNvramSpinBox.setSizePolicy(sizePolicy)
        self.uiNvramSpinBox.setMinimum(32)
        self.uiNvramSpinBox.setMaximum(65535)
        self.uiNvramSpinBox.setSingleStep(32)
        self.uiNvramSpinBox.setProperty("value", 128)
        self.uiNvramSpinBox.setObjectName("uiNvramSpinBox")
        self.gridLayout_2.addWidget(self.uiNvramSpinBox, 3, 1, 1, 1)
        self.verticalLayout_2.addWidget(self.uiOtherSettingsGroupBox)
        spacerItem = QtWidgets.QSpacerItem(260, 301, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.uiTabWidget.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.tab_2)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.groupBox = QtWidgets.QGroupBox(self.tab_2)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.uiEthernetAdaptersLabel = QtWidgets.QLabel(self.groupBox)
        self.uiEthernetAdaptersLabel.setObjectName("uiEthernetAdaptersLabel")
        self.gridLayout_3.addWidget(self.uiEthernetAdaptersLabel, 0, 0, 1, 1)
        self.uiEthernetAdaptersSpinBox = QtWidgets.QSpinBox(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiEthernetAdaptersSpinBox.sizePolicy().hasHeightForWidth())
        self.uiEthernetAdaptersSpinBox.setSizePolicy(sizePolicy)
        self.uiEthernetAdaptersSpinBox.setMaximum(16)
        self.uiEthernetAdaptersSpinBox.setSingleStep(2)
        self.uiEthernetAdaptersSpinBox.setObjectName("uiEthernetAdaptersSpinBox")
        self.gridLayout_3.addWidget(self.uiEthernetAdaptersSpinBox, 0, 1, 1, 1)
        self.uiSerialAdaptersLabel = QtWidgets.QLabel(self.groupBox)
        self.uiSerialAdaptersLabel.setObjectName("uiSerialAdaptersLabel")
        self.gridLayout_3.addWidget(self.uiSerialAdaptersLabel, 1, 0, 1, 1)
        self.uiSerialAdaptersSpinBox = QtWidgets.QSpinBox(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSerialAdaptersSpinBox.sizePolicy().hasHeightForWidth())
        self.uiSerialAdaptersSpinBox.setSizePolicy(sizePolicy)
        self.uiSerialAdaptersSpinBox.setMaximum(16)
        self.uiSerialAdaptersSpinBox.setSingleStep(2)
        self.uiSerialAdaptersSpinBox.setObjectName("uiSerialAdaptersSpinBox")
        self.gridLayout_3.addWidget(self.uiSerialAdaptersSpinBox, 1, 1, 1, 1)
        self.gridLayout_5.addWidget(self.groupBox, 0, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 347, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_5.addItem(spacerItem1, 1, 0, 1, 1)
        self.uiTabWidget.addTab(self.tab_2, "")
        self.verticalLayout.addWidget(self.uiTabWidget)

        self.retranslateUi(iouDeviceConfigPageWidget)
        self.uiTabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(iouDeviceConfigPageWidget)

    def retranslateUi(self, iouDeviceConfigPageWidget):
        _translate = QtCore.QCoreApplication.translate
        iouDeviceConfigPageWidget.setWindowTitle(_translate("iouDeviceConfigPageWidget", "IOU device configuration"))
        self.uiGeneralgroupBox.setTitle(_translate("iouDeviceConfigPageWidget", "General"))
        self.uiNameLabel.setText(_translate("iouDeviceConfigPageWidget", "Name:"))
        self.uiSymbolLabel.setText(_translate("iouDeviceConfigPageWidget", "Symbol:"))
        self.uiSymbolToolButton.setText(_translate("iouDeviceConfigPageWidget", "&Browse..."))
        self.uiCategoryLabel.setText(_translate("iouDeviceConfigPageWidget", "Category:"))
        self.uiIOUImageLabel.setText(_translate("iouDeviceConfigPageWidget", "IOU image path:"))
        self.uiIOUImageToolButton.setText(_translate("iouDeviceConfigPageWidget", "&Browse..."))
        self.uiStartupConfigLabel.setText(_translate("iouDeviceConfigPageWidget", "Startup-config:"))
        self.uiStartupConfigToolButton.setText(_translate("iouDeviceConfigPageWidget", "&Browse..."))
        self.uiPrivateConfigLabel.setText(_translate("iouDeviceConfigPageWidget", "Private-config:"))
        self.uiPrivateConfigToolButton.setText(_translate("iouDeviceConfigPageWidget", "&Browse..."))
        self.uiConsolePortLabel.setText(_translate("iouDeviceConfigPageWidget", "Console port:"))
        self.uiDefaultNameFormatLabel.setText(_translate("iouDeviceConfigPageWidget", "Default name format:"))
        self.uiOtherSettingsGroupBox.setTitle(_translate("iouDeviceConfigPageWidget", "Other settings"))
        self.uiL1KeepalivesCheckBox.setText(_translate("iouDeviceConfigPageWidget", "Enable layer 1 keepalive messages (testing only)"))
        self.uiDefaultValuesCheckBox.setText(_translate("iouDeviceConfigPageWidget", "Use default IOU values for memories"))
        self.uiRamLabel.setText(_translate("iouDeviceConfigPageWidget", "RAM size:"))
        self.uiRamSpinBox.setSuffix(_translate("iouDeviceConfigPageWidget", " MB"))
        self.uiNvramLabel.setText(_translate("iouDeviceConfigPageWidget", "NVRAM size:"))
        self.uiNvramSpinBox.setSuffix(_translate("iouDeviceConfigPageWidget", " KB"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.tab), _translate("iouDeviceConfigPageWidget", "General settings"))
        self.groupBox.setTitle(_translate("iouDeviceConfigPageWidget", "Adapters"))
        self.uiEthernetAdaptersLabel.setText(_translate("iouDeviceConfigPageWidget", "Ethernet adapters:"))
        self.uiEthernetAdaptersSpinBox.setToolTip(_translate("iouDeviceConfigPageWidget", "1 adapter equals 4 Ethernet interfaces"))
        self.uiSerialAdaptersLabel.setText(_translate("iouDeviceConfigPageWidget", "Serial adapters:"))
        self.uiSerialAdaptersSpinBox.setToolTip(_translate("iouDeviceConfigPageWidget", "1 adapter equals 4 serial interfaces"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.tab_2), _translate("iouDeviceConfigPageWidget", "Network"))




# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for Dynamips preferences.
"""

import os
import sys
import shutil
from gns3.qt import QtWidgets
from .. import Dynamips
from ..ui.dynamips_preferences_page_ui import Ui_DynamipsPreferencesPageWidget
from ..settings import DYNAMIPS_SETTINGS


class DynamipsPreferencesPage(QtWidgets.QWidget, Ui_DynamipsPreferencesPageWidget):

    """
    QWidget preference page for Dynamips.
    """

    def __init__(self):

        super().__init__()
        self.setupUi(self)

        # connect signals
        self.uiDynamipsPathToolButton.clicked.connect(self._dynamipsPathBrowserSlot)
        self.uiGhostIOSSupportCheckBox.stateChanged.connect(self._ghostIOSSupportSlot)
        self.uiRestoreDefaultsPushButton.clicked.connect(self._restoreDefaultsSlot)
        self.uiUseLocalServercheckBox.stateChanged.connect(self._useLocalServerSlot)

    def _dynamipsPathBrowserSlot(self):
        """
        Slot to open a file browser and select Dynamips executable.
        """

        file_filter = ""
        if sys.platform.startswith("win"):
            file_filter = "Executable (*.exe);;All files (*.*)"

        dynamips_path = shutil.which("dynamips")
        path, _ = QtWidgets.QFileDialog.getOpenFileName(self, "Select Dynamips", dynamips_path, file_filter)
        if not path:
            return

        if self._checkDynamipsPath(path):
            self.uiDynamipsPathLineEdit.setText(path)

    def _checkDynamipsPath(self, path):
        """
        Checks that the Dynamips path is valid.

        :param path: Dynamips path
        :returns: boolean
        """

        if not os.path.exists(path):
            QtWidgets.QMessageBox.critical(self, "Dynamips", '"{}" does not exist'.format(path))
            return False

        if not os.access(path, os.X_OK):
            QtWidgets.QMessageBox.critical(self, "Dynamips", "{} is not an executable".format(os.path.basename(path)))
            return False

        return True

    def _ghostIOSSupportSlot(self, state):
        """
        Slot to have the mmap checkBox checked if ghost IOS is checked.
        Ghost IOS is dependent on the mmap feature.

        :param state: state of the ghost IOS checkBox
        """

        if state:
            self.uiMmapSupportCheckBox.setChecked(True)

    def _restoreDefaultsSlot(self):
        """
        Slot to populate the page widgets with the default settings.
        """

        self._populateWidgets(DYNAMIPS_SETTINGS)

    def _useLocalServerSlot(self, state):
        """
        Slot to enable or not local server settings.
        """

        if state:
            self.uiDynamipsPathLineEdit.setEnabled(True)
            self.uiDynamipsPathToolButton.setEnabled(True)
            self.uiAllocateAuxConsolePortsCheckBox.setEnabled(True)
            self.uiGhostIOSSupportCheckBox.setEnabled(True)
            self.uiMmapSupportCheckBox.setEnabled(True)
            self.uiSparseMemorySupportCheckBox.setEnabled(True)
        else:
            self.uiDynamipsPathLineEdit.setEnabled(False)
            self.uiDynamipsPathToolButton.setEnabled(False)
            self.uiAllocateAuxConsolePortsCheckBox.setEnabled(False)
            self.uiGhostIOSSupportCheckBox.setEnabled(False)
            self.uiMmapSupportCheckBox.setEnabled(False)
            self.uiSparseMemorySupportCheckBox.setEnabled(False)

    def _populateWidgets(self, settings):
        """
        Populates the widgets with the settings.

        :param settings: Dynamips settings
        """

        self.uiDynamipsPathLineEdit.setText(settings["dynamips_path"])
        self.uiAllocateAuxConsolePortsCheckBox.setChecked(settings["allocate_aux_console_ports"])
        self.uiUseLocalServercheckBox.setChecked(settings["use_local_server"])
        self.uiGhostIOSSupportCheckBox.setChecked(settings["ghost_ios_support"])
        self.uiMmapSupportCheckBox.setChecked(settings["mmap_support"])
        self.uiSparseMemorySupportCheckBox.setChecked(settings["sparse_memory_support"])

    def loadPreferences(self):
        """
        Loads the Dynamips preferences.
        """

        dynamips_settings = Dynamips.instance().settings()
        self._populateWidgets(dynamips_settings)

    def savePreferences(self):
        """
        Saves the Dynamips preferences.
        """

        dynamips_path = self.uiDynamipsPathLineEdit.text().strip()
        if dynamips_path and self.uiUseLocalServercheckBox.isChecked() and not self._checkDynamipsPath(dynamips_path):
            return

        new_settings = {"dynamips_path": dynamips_path,
                        "allocate_aux_console_ports": self.uiAllocateAuxConsolePortsCheckBox.isChecked(),
                        "use_local_server": self.uiUseLocalServercheckBox.isChecked(),
                        "ghost_ios_support": self.uiGhostIOSSupportCheckBox.isChecked(),
                        "mmap_support": self.uiMmapSupportCheckBox.isChecked(),
                        "sparse_memory_support": self.uiSparseMemorySupportCheckBox.isChecked()}

        Dynamips.instance().setSettings(new_settings)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from gns3.network_client import getNetworkUrl


def test_getNetworkUrl():
    assert getNetworkUrl("http", "localhost", 8000) == "http://localhost:8000"
    assert getNetworkUrl("http", "localhost", 8000, "root") == "http://root@localhost:8000"
    # For a more friendly interface the user is hidden for autogenerated username
    assert getNetworkUrl("http", "localhost", 8000, "rsWfk49a15KipT9o8tx4vKmGRo6NtwYJ3Q2Yff6imf6PplX1hJWHJQ6ayi0XfdoF") == "http://localhost:8000"





# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/noplay/code/gns3/gns3-gui/gns3/modules/iou/ui/iou_preferences_page.ui'
#
# Created by: PyQt5 UI code generator 5.4.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_IOUPreferencesPageWidget(object):

    def setupUi(self, IOUPreferencesPageWidget):
        IOUPreferencesPageWidget.setObjectName("IOUPreferencesPageWidget")
        IOUPreferencesPageWidget.resize(400, 300)
        self.vboxlayout = QtWidgets.QVBoxLayout(IOUPreferencesPageWidget)
        self.vboxlayout.setObjectName("vboxlayout")
        self.uiTabWidget = QtWidgets.QTabWidget(IOUPreferencesPageWidget)
        self.uiTabWidget.setObjectName("uiTabWidget")
        self.uiGeneralSettingsTabWidget = QtWidgets.QWidget()
        self.uiGeneralSettingsTabWidget.setObjectName("uiGeneralSettingsTabWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.uiGeneralSettingsTabWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiLocalServerGroupBox = QtWidgets.QGroupBox(self.uiGeneralSettingsTabWidget)
        self.uiLocalServerGroupBox.setObjectName("uiLocalServerGroupBox")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.uiLocalServerGroupBox)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.uiIouyapPathLineEdit = QtWidgets.QLineEdit(self.uiLocalServerGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiIouyapPathLineEdit.sizePolicy().hasHeightForWidth())
        self.uiIouyapPathLineEdit.setSizePolicy(sizePolicy)
        self.uiIouyapPathLineEdit.setObjectName("uiIouyapPathLineEdit")
        self.horizontalLayout_6.addWidget(self.uiIouyapPathLineEdit)
        self.uiIouyapPathToolButton = QtWidgets.QToolButton(self.uiLocalServerGroupBox)
        self.uiIouyapPathToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiIouyapPathToolButton.setObjectName("uiIouyapPathToolButton")
        self.horizontalLayout_6.addWidget(self.uiIouyapPathToolButton)
        self.gridLayout_3.addLayout(self.horizontalLayout_6, 5, 0, 1, 2)
        self.uiIouyapPathLabel = QtWidgets.QLabel(self.uiLocalServerGroupBox)
        self.uiIouyapPathLabel.setObjectName("uiIouyapPathLabel")
        self.gridLayout_3.addWidget(self.uiIouyapPathLabel, 4, 0, 1, 1)
        self.uiLicensecheckBox = QtWidgets.QCheckBox(self.uiLocalServerGroupBox)
        self.uiLicensecheckBox.setChecked(True)
        self.uiLicensecheckBox.setObjectName("uiLicensecheckBox")
        self.gridLayout_3.addWidget(self.uiLicensecheckBox, 3, 0, 1, 1)
        self.uiUseLocalServercheckBox = QtWidgets.QCheckBox(self.uiLocalServerGroupBox)
        self.uiUseLocalServercheckBox.setChecked(True)
        self.uiUseLocalServercheckBox.setObjectName("uiUseLocalServercheckBox")
        self.gridLayout_3.addWidget(self.uiUseLocalServercheckBox, 0, 0, 1, 1)
        self.verticalLayout.addWidget(self.uiLocalServerGroupBox)
        self.uiAnyServerGroupBox = QtWidgets.QGroupBox(self.uiGeneralSettingsTabWidget)
        self.uiAnyServerGroupBox.setObjectName("uiAnyServerGroupBox")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.uiAnyServerGroupBox)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.uiIOURCPathLabel = QtWidgets.QLabel(self.uiAnyServerGroupBox)
        self.uiIOURCPathLabel.setObjectName("uiIOURCPathLabel")
        self.gridLayout_4.addWidget(self.uiIOURCPathLabel, 0, 0, 1, 1)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.uiIOURCPathLineEdit = QtWidgets.QLineEdit(self.uiAnyServerGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiIOURCPathLineEdit.sizePolicy().hasHeightForWidth())
        self.uiIOURCPathLineEdit.setSizePolicy(sizePolicy)
        self.uiIOURCPathLineEdit.setObjectName("uiIOURCPathLineEdit")
        self.horizontalLayout_5.addWidget(self.uiIOURCPathLineEdit)
        self.uiIOURCPathToolButton = QtWidgets.QToolButton(self.uiAnyServerGroupBox)
        self.uiIOURCPathToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiIOURCPathToolButton.setObjectName("uiIOURCPathToolButton")
        self.horizontalLayout_5.addWidget(self.uiIOURCPathToolButton)
        self.gridLayout_4.addLayout(self.horizontalLayout_5, 1, 0, 1, 1)
        self.verticalLayout.addWidget(self.uiAnyServerGroupBox)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.uiTabWidget.addTab(self.uiGeneralSettingsTabWidget, "")
        self.vboxlayout.addWidget(self.uiTabWidget)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem1 = QtWidgets.QSpacerItem(164, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.uiRestoreDefaultsPushButton = QtWidgets.QPushButton(IOUPreferencesPageWidget)
        self.uiRestoreDefaultsPushButton.setObjectName("uiRestoreDefaultsPushButton")
        self.horizontalLayout_2.addWidget(self.uiRestoreDefaultsPushButton)
        self.vboxlayout.addLayout(self.horizontalLayout_2)

        self.retranslateUi(IOUPreferencesPageWidget)
        self.uiTabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(IOUPreferencesPageWidget)

    def retranslateUi(self, IOUPreferencesPageWidget):
        _translate = QtCore.QCoreApplication.translate
        IOUPreferencesPageWidget.setWindowTitle(_translate("IOUPreferencesPageWidget", "IOS on UNIX"))
        self.uiLocalServerGroupBox.setTitle(_translate("IOUPreferencesPageWidget", "Local server"))
        self.uiIouyapPathToolButton.setText(_translate("IOUPreferencesPageWidget", "&Browse..."))
        self.uiIouyapPathLabel.setText(_translate("IOUPreferencesPageWidget", "Path to iouyap:"))
        self.uiLicensecheckBox.setText(_translate("IOUPreferencesPageWidget", "Check for a valid IOU license key"))
        self.uiUseLocalServercheckBox.setText(_translate("IOUPreferencesPageWidget", "Use the local server (Linux only)"))
        self.uiAnyServerGroupBox.setTitle(_translate("IOUPreferencesPageWidget", "Any server"))
        self.uiIOURCPathLabel.setText(_translate("IOUPreferencesPageWidget", "Path to IOURC (license, pushed to servers):"))
        self.uiIOURCPathToolButton.setText(_translate("IOUPreferencesPageWidget", "&Browse..."))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiGeneralSettingsTabWidget), _translate("IOUPreferencesPageWidget", "General settings"))
        self.uiRestoreDefaultsPushButton.setText(_translate("IOUPreferencesPageWidget", "Restore defaults"))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import uuid
import os
from unittest.mock import patch, MagicMock

from gns3.topology import Topology
from gns3.project import Project
from gns3.version import __version__
from gns3.items.pixmap_image_item import PixmapImageItem
import gns3.main_window


def test_topology_init():
    Topology()


def test_topology_node(vpcs_device):
    topology = Topology()
    topology.addNode(vpcs_device)
    assert len(topology.nodes()) == 1
    assert topology.getNode(vpcs_device.id()) == vpcs_device
    topology.removeNode(vpcs_device)
    assert len(topology.nodes()) == 0


def test_dump(vpcs_device, project, local_server):
    topology = Topology()
    topology.project = project
    topology.addNode(vpcs_device)

    dump = topology.dump(include_gui_data=False)
    assert dict(dump) == {
        "project_id": project.id(),
        "auto_start": False,
        "name": project.name(),
        "version": __version__,
        "revision": 4,
        "topology": {
            "nodes": [
                {
                    "description": "VPCS device",
                    "id": vpcs_device.id(),
                    "ports": [
                        {
                            "id": vpcs_device.ports()[0].id(),
                            "name": "Ethernet0",
                            "port_number": 0,
                            "adapter_number": 0
                        }
                    ],
                    "properties": {
                        "name": vpcs_device.name()
                    },
                    "server_id": local_server.id(),
                    "type": "VPCSDevice",
                    "vm_id": None
                }
            ],
            "servers": [
                {
                    "vm": False,
                    "host": "127.0.0.1",
                    "id": local_server.id(),
                    "local": True,
                    "port": 8000,
                    "protocol": "http",
                    "ram_limit": 0
                }
            ]
        },
        "type": "topology"
    }


def test_dump_http_auth(vpcs_device, project, remote_server):

    remote_server.setUser("hello")
    remote_server.setPassword("world")

    from gns3.modules.vpcs.vpcs_device import VPCSDevice
    from gns3.modules.vpcs import VPCS

    vpcs_device = VPCSDevice(VPCS(), remote_server, project)
    vpcs_device._vpcs_device_id = str(uuid.uuid4())
    vpcs_device._settings = {"name": "VPCS 1", "script_file": "", "console": None, "startup_script": None}
    vpcs_device.setInitialized(True)

    topology = Topology()
    topology.project = project
    topology.addNode(vpcs_device)

    dump = topology.dump(include_gui_data=False)
    assert dict(dump) == {
        "project_id": project.id(),
        "auto_start": False,
        "name": project.name(),
        "version": __version__,
        "revision": 4,
        "topology": {
            "nodes": [
                {
                    "description": "VPCS device",
                    "id": vpcs_device.id(),
                    "ports": [
                        {
                            "id": vpcs_device.ports()[0].id(),
                            "name": "Ethernet0",
                            "port_number": 0,
                            "adapter_number": 0
                        }
                    ],
                    "properties": {
                        "name": vpcs_device.name()
                    },
                    "server_id": remote_server.id(),
                    "type": "VPCSDevice",
                    "vm_id": None
                }
            ],
            "servers": [
                {
                    "vm": False,
                    "host": "127.0.0.1",
                    "id": remote_server.id(),
                    "local": False,
                    "port": 8001,
                    "protocol": "http",
                    "ram_limit": 0,
                    "user": "hello"
                }
            ]
        },
        "type": "topology"
    }


def test_randomize_id(project, tmpdir):
    project.setTopologyFile(str(tmpdir / "test.gns3"))

    project_uuid = str(uuid.uuid4())
    vm_uuid1 = str(uuid.uuid4())
    os.makedirs(str(tmpdir / "project-files" / "vpcs" / vm_uuid1))
    open(str(tmpdir / "project-files" / "vpcs" / vm_uuid1 / "test.log"), "w+").close()
    vm_uuid2 = str(uuid.uuid4())
    orig_topology = {
        "project_id": project_uuid,
        "topology": {
            "nodes": [
                {
                    "vm_id": vm_uuid1,
                },
                {
                    "vm_id": vm_uuid2
                }
            ]
        }
    }
    topology = Topology()
    topology.project = project
    top = topology._randomize_id(orig_topology)
    assert top["project_id"] != project_uuid
    assert top["topology"]["nodes"][0]["vm_id"] != vm_uuid1

    assert not os.path.exists(str(tmpdir / "project-files" / "vpcs" / vm_uuid1 / "test.log"))
    assert os.path.exists(str(tmpdir / "project-files" / "vpcs" / top["topology"]["nodes"][0]["vm_id"] / "test.log"))

    assert top["topology"]["nodes"][1]["vm_id"] != vm_uuid2
    assert top["topology"]["nodes"][0]["vm_id"] != top["topology"]["nodes"][1]["vm_id"]


def test_dump_random_id(vpcs_device, project, local_server):
    topology = Topology()
    topology.project = project
    topology.addNode(vpcs_device)

    fake_uuid = str(uuid.uuid4())
    with patch("uuid.uuid4", return_value=fake_uuid):
        dump = topology.dump(include_gui_data=False, random_id=True)
        assert dict(dump) == {
            "project_id": fake_uuid,
            "auto_start": False,
            "name": project.name(),
            "version": __version__,
            "revision": 4,
            "topology": {
                "nodes": [
                    {
                        "id": vpcs_device.id(),
                        "description": "VPCS device",
                        "ports": [
                            {
                                "id": vpcs_device.ports()[0].id(),
                                "name": "Ethernet0",
                                "port_number": 0,
                                "adapter_number": 0
                            }
                        ],
                        "properties": {
                            "name": vpcs_device.name()
                        },
                        "server_id": local_server.id(),
                        "type": "VPCSDevice",
                        "vm_id": fake_uuid}
                ],
                "servers": [
                    {
                        "host": "127.0.0.1",
                        "id": local_server.id(),
                        "local": True,
                        "port": 8000,
                        "protocol": "http",
                        "ram_limit": 0,
                        "vm": False
                    }
                ]
            },
            "type": "topology"
        }


def test_loadFile(tmpdir):
    topology = Topology()
    topo = str(tmpdir / "test" / "test.gns3")

    os.makedirs(str(tmpdir / "test"))
    with open(topo, 'w+') as f:
        f.write('{"name": "test", "type": "topology", "auto_start": false, "project_id": null, "topology": {}}')

    with patch("gns3.topology.Topology._load") as mock:
        project = Project()
        topology.loadFile(topo, project)

        assert mock.called
        args, kwargs = mock.call_args
        assert args[0] == {"name": "test", "auto_start": False, "project_id": None, "topology": {}, "type": "topology"}
        assert topology._project.filesDir() == str(tmpdir / "test")
        assert topology._project.name() == "test"


def test_load(project, monkeypatch, main_window, tmpdir):

    topo = {
        "project_id": project.id(),
        "auto_start": False,
        "name": "twovpcs",
        "topology": {
            "links": [
                {
                    "description": "Link from VPCS 1 port Ethernet0 to VPCS 2 port Ethernet0",
                    "destination_node_id": 2,
                    "destination_port_id": 2,
                    "id": 1,
                    "source_node_id": 1,
                    "source_port_id": 1
                }
            ],
            "nodes": [
                {
                    "description": "VPCS device",
                    "id": 1,
                    "label": {
                        "color": "#000000",
                        "font": "TypeWriter,10,-1,5,75,0,0,0,0,0",
                        "text": "VPCS 1",
                        "x": 10.75,
                        "y": -25.0
                    },
                    "ports": [
                        {
                            "description": "connected to VPCS 2 on port Ethernet0",
                            "id": 1,
                            "link_id": 1,
                            "name": "Ethernet0",
                            "nio": "NIO_UDP",
                            "port_number": 0,
                            "adapter_number": 0
                        }
                    ],
                    "properties": {
                        "console": 4501,
                        "name": "VPCS 1",
                        "script_file": "startup.vpc"
                    },
                    "server_id": 1,
                    "type": "VPCSDevice",
                    "vpcs_id": 1,
                    "vm_id": "2b5476de-6e79-4eb5-b0eb-8c54c7821cb8",
                    "x": -349.5,
                    "y": -206.5
                },
                {
                    "description": "VPCS device",
                    "id": 2,
                    "label": {
                        "color": "#000000",
                        "font": "TypeWriter,10,-1,5,75,0,0,0,0,0",
                        "text": "VPCS 2",
                        "x": 10.75,
                        "y": -25.0
                    },
                    "ports": [
                        {
                            "description": "connected to VPCS 1 on port Ethernet0",
                            "id": 2,
                            "link_id": 1,
                            "name": "Ethernet0",
                            "nio": "NIO_UDP",
                            "port_number": 0
                        }
                    ],
                    "properties": {
                        "console": 4502,
                        "name": "VPCS 2",
                        "script_file": "startup.vpc"
                    },
                    "server_id": 1,
                    "type": "VPCSDevice",
                    "vm_id": "2b5476de-6e79-4eb5-b0eb-8c54c7821cba",
                    "vpcs_id": 2,
                    "x": 69.5,
                    "y": -190.5
                }
            ],
            "servers": [
                {
                    "cloud": False,
                    "host": "127.0.0.1",
                    "id": 1,
                    "local": True,
                    "port": 8000
                }
            ]
        },
        "type": "topology",
        "version": "1.3.0"
    }

    monkeypatch.setattr('gns3.main_window.MainWindow.instance', lambda: main_window)

    # We return an uuid for each HTTP post
    def http_loader(self, method, path, callback, body={}, **kwargs):
        if path == "/projects":
            callback({"project_id": uuid.uuid4(), "path": str(tmpdir)})
        elif path[-14:] == "/notifications":
            pass
        else:
            callback({"vm_id": uuid.uuid4()})

    monkeypatch.setattr("gns3.http_client.HTTPClient.createHTTPQuery", http_loader)

    monkeypatch.setattr("gns3.http_client.HTTPClient.connected", lambda self: True)

    topology = Topology()
    topology.project = project
    topology._load(topo)

    assert topology._project.id() == project.id()
    assert len(topology.nodes()) == 2
    assert len(topology._node_to_links_mapping) == 2
    assert topology.getNode(1).initialized()
    assert topology.getNode(2).initialized()
    assert main_window.uiGraphicsView.addLink.called


def test_load_1_2_topology(project, monkeypatch, main_window, tmpdir):

    topo = {
        "auto_start": False,
        "name": "twovpcs",
        "topology": {
            "links": [
                {
                    "description": "Link from VPCS 1 port Ethernet0 to VPCS 2 port Ethernet0",
                    "destination_node_id": 2,
                    "destination_port_id": 2,
                    "id": 1,
                    "source_node_id": 1,
                    "source_port_id": 1
                }
            ],
            "nodes": [
                {
                    "description": "VPCS device",
                    "id": 1,
                    "label": {
                        "color": "#000000",
                        "font": "TypeWriter,10,-1,5,75,0,0,0,0,0",
                        "text": "VPCS 1",
                        "x": 10.75,
                        "y": -25.0
                    },
                    "ports": [
                        {
                            "description": "connected to VPCS 2 on port Ethernet0",
                            "id": 1,
                            "link_id": 1,
                            "name": "Ethernet0",
                            "nio": "NIO_UDP",
                            "port_number": 0
                        }
                    ],
                    "properties": {
                        "console": 4501,
                        "name": "VPCS 1",
                        "script_file": "startup.vpc"
                    },
                    "server_id": 1,
                    "type": "VPCSDevice",
                    "vpcs_id": 1,
                    "x": -349.5,
                    "y": -206.5
                },
                {
                    "description": "VPCS device",
                    "id": 2,
                    "label": {
                        "color": "#000000",
                        "font": "TypeWriter,10,-1,5,75,0,0,0,0,0",
                        "text": "VPCS 2",
                        "x": 10.75,
                        "y": -25.0
                    },
                    "ports": [
                        {
                            "description": "connected to VPCS 1 on port Ethernet0",
                            "id": 2,
                            "link_id": 1,
                            "name": "Ethernet0",
                            "nio": "NIO_UDP",
                            "port_number": 0
                        }
                    ],
                    "properties": {
                        "console": 4502,
                        "name": "VPCS 2",
                        "script_file": "startup.vpc"
                    },
                    "server_id": 1,
                    "type": "VPCSDevice",
                    "vpcs_id": 2,
                    "x": 69.5,
                    "y": -190.5
                }
            ],
            "servers": [
                {
                    "cloud": False,
                    "host": "127.0.0.1",
                    "id": 1,
                    "local": True,
                    "port": 8000
                }
            ]
        },
        "type": "topology",
        "version": "1.2.3"
    }

    monkeypatch.setattr('gns3.main_window.MainWindow.instance', lambda: main_window)

    project_call = 0
    # We return an uuid for each HTTP post

    def http_loader(self, method, path, callback, body={}, **kwargs):
        if path == "/projects":
            callback({"project_id": uuid.uuid4(), "path": str(tmpdir)}, error=False, server=local_server)
            project_call += 1
            assert project_call < 2
        elif path[-14:] == "/notifications":
            pass
        else:
            callback({"vm_id": uuid.uuid4()})

    monkeypatch.setattr("gns3.http_client.HTTPClient.createHTTPQuery", http_loader)

    monkeypatch.setattr("gns3.http_client.HTTPClient.connected", lambda self: True)

    topology = Topology()
    topology.project = project
    topology._project.setName("unsaved")
    topology._project.setTopologyFile(str(tmpdir))
    topology.dump = MagicMock()
    topology._load(topo)

    assert topology._project.id() is not None
    assert len(topology.nodes()) == 2
    assert len(topology._node_to_links_mapping) == 2
    assert topology.getNode(1).initialized()
    assert topology.getNode(2).initialized()
    assert main_window.uiGraphicsView.addLink.called
    assert main_window.saveProject.called


def test_image_in_project(tmpdir):
    project_dir = tmpdir / "project" / "project-files"
    os.makedirs(str(project_dir / "images"))

    project = Project()
    project.setFilesDir(str(tmpdir / "project"))

    topology = Topology()
    topology.project = project

    with open(str(project_dir / "images" / "1.jpg"), "w+") as f:
        f.write("AA")

    image1 = PixmapImageItem(None, "images/1.jpg")
    topology.addImage(image1)
    assert image1 in topology._images

    image2 = PixmapImageItem(None, "images/1.jpg")
    topology.addImage(image2)
    assert image1 in topology._images
    assert image2 in topology._images

    topology.removeImage(image2)
    assert os.path.exists(str(project_dir / "images" / "1.jpg"))

    # If not image use this file delete it
    topology.removeImage(image1)
    assert not os.path.exists(str(project_dir / "images" / "1.jpg"))


def test_image_outside_project(tmpdir):
    """
    By security we do not delete image outside project.
    This should not append but if someone reuse the image items for
    something else.
    """

    project_img_dir = tmpdir / "project" / "project-files" / "images"
    os.makedirs(str(project_img_dir))

    project = Project()
    project.setFilesDir(str(tmpdir / "project"))

    topology = Topology()
    topology.project = project

    with open(str(tmpdir / "1.jpg"), "w+") as f:
        f.write("AA")

    image1 = PixmapImageItem(None, str(tmpdir / "1.jpg"))
    topology.addImage(image1)
    assert image1 in topology._images
    topology.removeImage(image1)
    assert os.path.exists(str(tmpdir / "1.jpg"))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Manages and stores everything needed for a connection between 2 devices.
"""


from .qt import QtCore
from .nios.nio_udp import NIOUDP
from .nios.nio_vmnet import NIOVMNET

import logging
log = logging.getLogger(__name__)


class Link(QtCore.QObject):

    """
    Link implementation.

    :param source_node: source Node instance
    :param source_port: source Port instance
    :param destination_node: destination Node instance
    :param destination_port: destination Port instance
    :param stub: indicates if the link is connected to a stub device like a Cloud
    """

    # signals used to let the GUI view know about link
    # additions and deletions.
    add_link_signal = QtCore.Signal(int)
    delete_link_signal = QtCore.Signal(int)

    _instance_count = 1

    def __init__(self, source_node, source_port, destination_node, destination_port):

        super().__init__()

        log.info("adding link from {} {} to {} {}".format(source_node.name(),
                                                          source_port.name(),
                                                          destination_node.name(),
                                                          destination_port.name()))

        # create an unique ID
        self._id = Link._instance_count
        Link._instance_count += 1

        self._source_node = source_node
        self._source_port = source_port
        self._destination_node = destination_node
        self._destination_port = destination_port
        self._source_nio = None
        self._destination_nio = None
        self._source_nio_active = False
        self._destination_nio_active = False

        if source_port.isStub() or destination_port.isStub():
            self._stub = True
        else:
            self._stub = False
        # we must request UDP information if the NIO is a NIO UDP and before
        # it can be created.
        if not self._stub:
            # connect signals used when a NIO has been created by a node
            # and this NIO need to be attached to a port connected to this link
            source_node.nio_signal.connect(self.newNIOSlot)
            destination_node.nio_signal.connect(self.newNIOSlot)

            # currently, we support only NIO_UDP and NIO_VMNET for normal connections (non-stub).
            if source_port.defaultNio() == NIOUDP:
                assert destination_port.defaultNio() == NIOUDP
                self._source_udp = None
                self._destination_udp = None

                # connect signals used to receive a UDP port and host allocated by a node
                source_node.allocate_udp_nio_signal.connect(self.UDPPortAllocatedSlot)
                destination_node.allocate_udp_nio_signal.connect(self.UDPPortAllocatedSlot)

                # request the UDP info for each node
                source_node.allocateUDPPort(self._source_port.id())
                destination_node.allocateUDPPort(self._destination_port.id())
            elif source_port.defaultNio() == NIOVMNET:
                assert destination_port.defaultNio() == NIOVMNET
                source_node.allocate_vmnet_nio_signal.connect(self.VMnetInterfaceAllocatedSlot)
                source_node.allocateVMnetInterface(self._source_port.id())
            else:
                raise NotImplementedError()
        else:
            # handle stub connections (to a cloud for instance).
            if not source_port.isStub() and destination_port.isStub():
                source_node.nio_signal.connect(self.newNIOSlot)
                self._source_nio = self._destination_port.defaultNio()
                self._source_node.nio_cancel_signal.connect(self.cancelNIOSlot)
                self._source_node.addNIO(self._source_port, self._source_nio)
            elif not destination_port.isStub() and source_port.isStub():
                destination_node.nio_signal.connect(self.newNIOSlot)
                self._destination_nio = self._source_port.defaultNio()
                self._destination_node.nio_cancel_signal.connect(self.cancelNIOSlot)
                self._destination_node.addNIO(self._destination_port, self._destination_nio)
            else:
                log.error("both ports are stub!")

    @classmethod
    def reset(cls):
        """
        Reset the instance count.
        """

        cls._instance_count = 1

    def __str__(self):

        return "Link from {} port {} to {} port {}".format(self._source_node.name(),
                                                           self._source_port.name(),
                                                           self._destination_node.name(),
                                                           self._destination_port.name())

    def deleteLink(self):
        """
        Deletes this link.
        """

        log.info("deleting link from {} {} to {} {}".format(self._source_node.name(),
                                                            self._source_port.name(),
                                                            self._destination_node.name(),
                                                            self._destination_port.name()))

        # delete the NIOs on both source and destination nodes
        if self._source_port.nio():
            self._source_node.deleteNIO(self._source_port)
        self._source_port.setFree()
        self._source_node.updated_signal.emit()
        if self._destination_port.nio():
            self._destination_node.deleteNIO(self._destination_port)
        self._destination_port.setFree()
        self._destination_node.updated_signal.emit()

        # let the GUI know about this link has been deleted
        self.delete_link_signal.emit(self._id)

    def id(self):
        """
        Returns this link identifier.

        :returns: link identifier (integer)
        """

        return self._id

    def sourceNode(self):
        """
        Returns the source node for this link.

        :returns: Node instance
        """

        return self._source_node

    def destinationNode(self):
        """
        Returns the destination node for this link.

        :returns: Node instance
        """

        return self._destination_node

    def sourcePort(self):
        """
        Returns the source port for this link.

        :returns: Port instance
        """

        return self._source_port

    def destinationPort(self):
        """
        Returns the destination port for this link.

        :returns: Port instance
        """

        return self._destination_port

    def UDPPortAllocatedSlot(self, node_id, port_id, lport):
        """
        Slot to receive events from Node instances
        when a UDP port has been allocated in order to create a NIO UDP.

        :param node_id: node identifier
        :param port_id: port identifier
        :param lport: local UDP port
        """
        # check that the node is connected to this link as a source
        if node_id == self._source_node.id() and port_id == self._source_port.id():
            laddr = self._source_node.server().host()
            self._source_udp = (lport, laddr)
            # disconnect the signal has we don't expect new source UDP info for this link.
            self._source_node.allocate_udp_nio_signal.disconnect(self.UDPPortAllocatedSlot)

            log.debug("{} has allocated UDP port {} for host {}".format(self._source_node.name(),
                                                                        lport,
                                                                        laddr))

        # check that the node is connected to this link as a destination
        elif node_id == self._destination_node.id() and port_id == self._destination_port.id():
            laddr = self._destination_node.server().host()
            self._destination_udp = (lport, laddr)
            # disconnect the signal has we don't expect new source UDP info for this link.
            self._destination_node.allocate_udp_nio_signal.disconnect(self.UDPPortAllocatedSlot)

            log.debug("{} has allocated UDP port {} for host {}".format(self._destination_node.name(),
                                                                        lport,
                                                                        laddr))

        if self._source_udp and self._destination_udp:
            # we got UDP info from both source and destination nodes
            # meaning we can proceed with the creation of UDP NIOs
            lport, laddr = self._source_udp
            rport, raddr = self._destination_udp

            self._source_nio = NIOUDP(lport, raddr, rport)
            self._destination_nio = NIOUDP(rport, laddr, lport)

            self._source_udp = None
            self._destination_udp = None

            log.debug("creating UDP tunnel from {}:{} to {}:{} ".format(laddr, lport, raddr, rport))

            # add the UDP NIOs to the nodes
            self._source_node.nio_cancel_signal.connect(self.cancelNIOSlot)
            self._source_node.addNIO(self._source_port, self._source_nio)
            self._destination_node.nio_cancel_signal.connect(self.cancelNIOSlot)
            self._destination_node.addNIO(self._destination_port, self._destination_nio)

    def VMnetInterfaceAllocatedSlot(self, node_id, port_id, vmnet):
        """
        Slot to receive events from Node instances
        when a VMnet interface has been allocated in order to create a NIO VMNET.

        :param node_id: node identifier
        :param port_id: port identifier
        :param vmnet: vmnet interface name
        """

        # check that the node is connected to this link as a source
        # only the source is used to request the server for a vmnet interface
        # and then allocate a NIO VMNET to both the source and destination
        if node_id == self._source_node.id() and port_id == self._source_port.id():
            self._source_node.allocate_vmnet_nio_signal.disconnect(self.VMnetInterfaceAllocatedSlot)
            self._source_nio = NIOVMNET(vmnet)
            self._destination_nio = NIOVMNET(vmnet)

            # add the VMnet NIOs to the nodes
            self._source_node.nio_cancel_signal.connect(self.cancelNIOSlot)
            self._source_node.addNIO(self._source_port, self._source_nio)
            self._destination_node.nio_cancel_signal.connect(self.cancelNIOSlot)
            self._destination_node.addNIO(self._destination_port, self._destination_nio)

    def newNIOSlot(self, node_id, port_id):
        """
        Slot to receive events from Node instances
        when a NIO has been created on the server
        and are active.

        :param node_id: node identifier
        :param port_id: port identifier
        """

        # in very rare cases link is already deleted
        if self is None:
            return

        # check that the node is connected to this link as a source
        if node_id == self._source_node.id() and port_id == self._source_port.id():
            self._source_nio_active = True
            # disconnect the signal has we don't expect new source NIO for this link.
            self._source_node.nio_signal.disconnect(self.newNIOSlot)

        # check that the node is connected to this link as a destination
        elif node_id == self._destination_node.id() and port_id == self._destination_port.id():
            self._destination_nio_active = True
            # disconnect the signal has we don't expect new destination NIO for this link.
            self._destination_node.nio_signal.disconnect(self.newNIOSlot)

        if not self._stub and self._source_nio_active and self._destination_nio_active:
            # both NIOs are active now.
            self._addToSourcePort(self._source_nio)
            self._addToDestinationPort(self._destination_nio)

            self._source_node.nio_cancel_signal.disconnect(self.cancelNIOSlot)
            self._destination_node.nio_cancel_signal.disconnect(self.cancelNIOSlot)
            self._source_nio_active = False
            self._destination_nio_active = False

            # let the GUI know about this link has been created
            self.add_link_signal.emit(self._id)
        elif self._stub and self._source_nio_active:
            self._addToSourcePort(self._source_nio)
            # add the NIO to destination to show the port is not free.
            self._addToDestinationPort(self._source_nio)
            self._source_nio_active = False
            self._source_node.nio_cancel_signal.disconnect(self.cancelNIOSlot)
            self.add_link_signal.emit(self._id)
        elif self._stub and self._destination_nio_active:
            # add the NIO to source to show the port is not free.
            self._addToSourcePort(self._destination_nio)
            self._addToDestinationPort(self._destination_nio)
            self._destination_nio_active = False
            self._destination_node.nio_cancel_signal.disconnect(self.cancelNIOSlot)
            self.add_link_signal.emit(self._id)

    def _addToSourcePort(self, nio):
        """
        Adds a NIO, a link id and a description to the source port.

        :param nio: NIO instance
        """

        self._source_port.setNio(nio)
        self._source_port.setLinkId(self._id)
        self._source_port.setDestinationNode(self._destination_node)
        self._source_port.setDestinationPort(self._destination_port)

        log.debug("{} attached to {} on port {}".format(nio,
                                                        self._source_node.name(),
                                                        self._source_port.name()))

    def _addToDestinationPort(self, nio):
        """
        Adds a NIO, a link id and a description to the destination port.

        :param nio: NIO instance
        """

        self._destination_port.setNio(nio)
        self._destination_port.setLinkId(self._id)
        self._destination_port.setDestinationNode(self._source_node)
        self._destination_port.setDestinationPort(self._source_port)

        log.debug("{} attached to {} on port {}".format(nio,
                                                        self._destination_node.name(),
                                                        self._destination_port.name()))

    def cancelNIOSlot(self, node_id):
        """
        Slot to receive events from Node instances
        when a NIO has been canceled because of an
        error returned by the server.

        :param node_id: node identifier
        """

        if not self._stub:
            try:
                # the destination node has canceled its NIO allocation
                self._destination_node.nio_signal.disconnect(self.newNIOSlot)
            except TypeError:
                # ignore TypeError: 'method' object is not connected
                pass

            try:
                # the source node has canceled its NIO allocation
                self._source_node.nio_signal.disconnect(self.newNIOSlot)
            except TypeError:
                # ignore TypeError: 'method' object is not connected
                pass

            self._source_node.nio_cancel_signal.disconnect(self.cancelNIOSlot)
            self._destination_node.nio_cancel_signal.disconnect(self.cancelNIOSlot)
        else:
            if self._source_node.id() == node_id:
                self._source_node.nio_signal.disconnect(self.newNIOSlot)
                self._source_node.nio_cancel_signal.disconnect(self.cancelNIOSlot)
            else:
                self._destination_node.nio_signal.disconnect(self.newNIOSlot)
                self._destination_node.nio_cancel_signal.disconnect(self.cancelNIOSlot)

        self._source_nio_active = False
        self._destination_nio_active = False
        self.deleteLink()

    def dump(self):
        """
        Returns a representation of this link.

        :returns: dictionary
        """

        return {"id": self.id(),
                "description": str(self),
                "source_node_id": self._source_node.id(),
                "source_port_id": self._source_port.id(),
                "destination_node_id": self._destination_node.id(),
                "destination_port_id": self._destination_port.id()}
